/*!
 * Tine 2.0 - Expressodriver 
 * Copyright (c) 2007-2011 Metaways Infosystems GmbH (http://www.metaways.de)
 * http://www.gnu.org/licenses/agpl.html AGPL Version 3
 */
/*
 * Tine 2.0
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 *
 */

Ext.ns('Tine.Expressodriver');

/**
 * generic exception handler for expressodriver
 *
 * @namespace Tine.Expressodriver
 * @param {Tine.Exception} exception
 * @param {Object} request
 */
Tine.Expressodriver.handleRequestException = function(exception, request) {

    var app = Tine.Tinebase.appMgr.get('Expressodriver'),
        existingFilenames = [],
        nonExistantFilenames = [],
        i,
        filenameWithoutPath = null;

    switch(exception.code) {
        // overwrite default 503 handling and add a link to the wiki
        case 503:
            Ext.MessageBox.show({
                buttons: Ext.Msg.OK,
                icon: Ext.MessageBox.WARNING,
                title: _('Service Unavailable'),
                msg: String.format(app.i18n._('The Expressodriver is not configured correctly. Please refer to the {0}Tine 2.0 Admin FAQ{1} for configuration advice or contact your administrator.'),
                    '<a href="http://www.tine20.org/wiki/index.php/Admin_FAQ#The_message_.22filesdir_config_value_not_set.22_appears_in_the_logfile_and_I_can.27t_open_the_Expressodriver" target="_blank">',
                    '</a>')
            });
            break;

        case 901:
            if (request) {
                Tine.log.debug('Tine.Expressodriver.handleRequestException - request exception:');
                Tine.log.debug(exception);

                if (exception.existingnodesinfo) {
                    for (i = 0; i < exception.existingnodesinfo.length; i++) {
                        existingFilenames.push(exception.existingnodesinfo[i].name);
                    }
                }

                this.conflictConfirmWin = Tine.widgets.dialog.FileListDialog.openWindow({
                    modal: true,
                    allowCancel: false,
                    height: 180,
                    width: 300,
                    title: app.i18n._('Files already exists') + '. ' + app.i18n._('Do you want to replace the following file(s)?'),
                    text: existingFilenames.join('<br />'),
                    scope: this,
                    handler: function(button) {
                        var params = request.params,
                            uploadKey = exception.uploadKeyArray;
                        params.method = request.method;
                        params.forceOverwrite = true;

                        if (button == 'no') {
                            if (params.method == 'Expressodriver.moveNodes') {
                                // reload grid
                                var app = Tine.Tinebase.appMgr.get('Expressodriver');
                                app.getMainScreen().getCenterPanel().grid.getStore().reload();
                                // do nothing, other nodes has been moved already
                                return;
                            }

                            Tine.log.debug('Tine.Expressodriver.handleRequestException::' + params.method + ' -> only non-existant nodes.');

                            Ext.each(params.filenames, function(filename) {
                                filenameWithoutPath = filename.match(/[^\/]*$/);
                                if (filenameWithoutPath && existingFilenames.indexOf(filenameWithoutPath[0]) === -1) {
                                    nonExistantFilenames.push(filename);
                                }
                            });

                            params.filenames = nonExistantFilenames;

                            uploadKey = nonExistantFilenames;
                        } else {
                            Tine.log.debug('Tine.Expressodriver.handleRequestException::' + params.method + ' -> replace all existing nodes.');
                        }

                        if (params.method == 'Expressodriver.copyNodes' || params.method == 'Expressodriver.moveNodes' ) {
                            Tine.Expressodriver.fileRecordBackend.copyNodes(null, null, null, params);
                        } else if (params.method == 'Expressodriver.createNodes' ) {
                            Tine.Expressodriver.fileRecordBackend.createNodes(params, uploadKey, exception.addToGridStore);
                        }
                    }
                });

            } else {
                Ext.Msg.show({
                  title:   app.i18n._('Failure on create folder'),
                  msg:     app.i18n._('Item with this name already exists!'),
                  icon:    Ext.MessageBox.ERROR,
                  buttons: Ext.Msg.OK
               });
            }
            break;
        case 902: // Expressodriver_Exception_DestinationIsOwnChild
            Ext.MessageBox.show({
                buttons: Ext.Msg.OK,
                icon: Ext.MessageBox.ERROR,
                title: app.i18n._(exception.title),
                msg: app.i18n._(exception.message)
            });
            break;
        case 903: // Expressodriver_Exception_DestinationIsSameNode
            Ext.MessageBox.show({
                buttons: Ext.Msg.OK,
                icon: Ext.MessageBox.INFO,
                title: app.i18n._(exception.title),
                msg: app.i18n._(exception.message)
            });
            break;
        default:
            Tine.Tinebase.ExceptionHandler.handleRequestException(exception);
            break;
    }
};/*
 * Tine 2.0
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 */
Ext.ns('Tine.Expressodriver');

/**
 * Grid context menu
 */
Tine.Expressodriver.GridContextMenu = {
    /**
     * create tree node
     */
    addNode: function() {
        Tine.log.debug("grid add");
    },

    /**
     * rename tree node
     */
    renameNode: function() {
        if (this.scope.ctxNode) {

            var node = this.scope.ctxNode[0];

            var nodeText = node.data.name;
            if(typeof nodeText == 'object') {
                nodeText = nodeText.name;
            }

            Ext.MessageBox.show({
                title: this.scope.app.i18n._('Rename') + ' ' + this.nodeName,
                msg: String.format(_('Please enter the new name of the {0}:'), this.nodeName),
                buttons: Ext.MessageBox.OKCANCEL,
                value: nodeText,
                fn: function(_btn, _text){
                    if (_btn == 'ok') {
                        if (! _text) {
                            Ext.Msg.alert(String.format(this.scope.app.i18n._('Not renamed {0}'), this.nodeName), String.format(_('You have to supply a {0} name!'), this.nodeName));
                            return;
                        }
                        if (_text === nodeText) {
                            Ext.Msg.alert(String.format(this.scope.app.i18n._('Not renamed {0}'), this.nodeName), this.scope.app.i18n._('You have to supply a different name!'));
                            return;
                        }

                        var params = {
                                method: this.backend + '.rename' + this.backendModel,
                                newName: _text
                        };

                        if (this.backendModel == 'Node') {
                            params.application = this.scope.app.appName || this.scope.appName;
                            var filename = node.data.path;
                            params.sourceFilenames = [filename];

                            var targetFilename = "/";
                            var sourceSplitArray = filename.split("/");
                            for (var i=1; i<sourceSplitArray.length-1; i++) {
                                targetFilename += sourceSplitArray[i] + '/';
                            }

                            params.destinationFilenames = [targetFilename + _text];
                            params.method = this.backend + '.moveNodes';
                        }
                        Ext.MessageBox.wait(_('Please wait'), this.scope.app.i18n._('Renaming nodes...' ));
                        Ext.Ajax.request({
                            params: params,
                            scope: this,
                            success: function(_result, _request){
                                var nodeData = Ext.util.JSON.decode(_result.responseText)[0];
                                this.scope.fireEvent('containerrename', nodeData);

                                // TODO: im event auswerten
                                if (this.backendModel == 'Node') {
                                    var grid = this.scope.app.getMainScreen().getCenterPanel();
                                    grid.getStore().reload();

                                    var nodeName = nodeData.name;
                                    if(typeof nodeName == 'object') {
                                        nodeName = nodeName.name;
                                    }

                                    var treeNode = this.scope.app.getMainScreen().getWestPanel().getContainerTreePanel().getNodeById(nodeData.id);
                                    if(treeNode) {
                                        treeNode.setText(nodeName);
                                        treeNode.attributes.nodeRecord.beginEdit();
                                        treeNode.attributes.nodeRecord.set('name', nodeName); // TODO set path
                                        treeNode.attributes.nodeRecord.set('path', nodeData.path); // TODO set path
                                        treeNode.attributes.path = nodeData.path; // TODO set path
                                        treeNode.attributes.nodeRecord.commit(false);

                                        if(typeof treeNode.attributes.name == 'object') {
                                            treeNode.attributes.name.name = nodeName; // TODO set path
                                        }
                                        else {
                                            treeNode.attributes.name = nodeName;
                                        }
                                    }
                                }
                                Ext.MessageBox.hide();
                            },
                            failure: function(result, request) {
                                var nodeData = Ext.util.JSON.decode(result.responseText);

                                var appContext = Tine[this.scope.app.appName];
                                if(appContext && appContext.handleRequestException) {
                                    appContext.handleRequestException(nodeData.data);
                                }
                            }
                        });
                    }
                },
                scope: this,
                prompt: true,
                icon: Ext.MessageBox.QUESTION
            });
        }
    },

    /**
     * delete tree node
     */
    deleteNode: function() {
        if (this.scope.ctxNode) {
            var nodes = this.scope.ctxNode;

            var nodeName = "";
            if(nodes && nodes.length) {
                for(var i=0; i<nodes.length; i++) {
                    var currNodeData = nodes[i].data;

                    if(typeof currNodeData.name == 'object') {
                        nodeName += currNodeData.name.name + '<br />';
                    }
                    else {
                        nodeName += currNodeData.name + '<br />';
                    }
                }

            }

            this.conflictConfirmWin = Tine.widgets.dialog.FileListDialog.openWindow({
                modal: true,
                allowCancel: false,
                height: 180,
                width: 300,
                title: this.scope.app.i18n._('Do you really want to delete the following files?'),
                text: nodeName,
                scope: this,
                handler: function(button){
                    if (button == 'yes') {
                        var params = {
                                method: this.backend + '.delete' + this.backendModel
                        };

                        if (this.backendModel == 'Node') {

                            var filenames = new Array();
                            if(nodes) {
                                for(var i=0; i<nodes.length; i++) {
                                    filenames.push(nodes[i].data.path);
                                }
                            }
                            params.application = this.scope.app.appName || this.scope.appName;
                            params.filenames = filenames;
                            params.method = this.backend + ".deleteNodes";
                        }
                        Ext.MessageBox.wait(_('Please wait'), this.scope.app.i18n._('Deleting nodes...' ));
                        Ext.Ajax.request({
                            params: params,
                            scope: this,
                            success: function(_result, _request){

                                if(nodes &&  this.backendModel == 'Node') {
                                    var treePanel = this.scope.app.getMainScreen().getWestPanel().getContainerTreePanel();
                                    for(var i=0; i<nodes.length; i++){
                                        treePanel.fireEvent('containerdelete', nodes[i].data.container_id);
                                        // TODO: in EventHandler auslagern
                                        var treeNode = treePanel.getNodeById(nodes[i].id);
                                        if(treeNode) {
                                            treeNode.parentNode.removeChild(treeNode);
                                        }
                                    }
                                    for(var i=0; i<nodes.length; i++) {
                                        var node = nodes[i];
                                        if(node.fileRecord) {
                                            var upload = Tine.Tinebase.uploadManager.getUpload(node.fileRecord.get('uploadKey'));
                                            upload.setPaused(true);
                                            Tine.Tinebase.uploadManager.unregisterUpload(upload.id);
                                        }
                                    }
                                    this.scope.app.getMainScreen().getCenterPanel().getStore().remove(nodes);
                                }
                                Ext.MessageBox.hide();
                            },
                            failure: function(result, request) {
                                var nodeData = Ext.util.JSON.decode(result.responseText);

                                var appContext = Tine[this.scope.app.appName];
                                if(appContext && appContext.handleRequestException) {
                                    appContext.handleRequestException(nodeData.data);
                                }
                            }
                        });
                    }

                }
            });

        }
    },

    /**
     * change tree node color
     */
    changeNodeColor: function(cp, color) {
        Tine.log.debug("grid change color");


    },

    /**
     * manage permissions
     *
     */
    managePermissions: function() {
        Tine.log.debug("grid manage permissions");
    },

    /**
     * reload node
     */
    reloadNode: function() {
        Tine.log.debug("grid reload node");
    },

    /**
     * calls the file edit dialog from the grid
     * @param {} button
     * @param {} event
     */
    onEditFile: function(button, event) {
        var app = Tine.Tinebase.appMgr.get('Expressodriver');
        var grid = app.getMainScreen().getCenterPanel();
        grid.onEditFile.call(grid);
    },

    /**
     * download file
     *
     * @param {} button
     * @param {} event
     */
    downloadFile: function(button, event) {

        var grid = this.scope.app.getMainScreen().getCenterPanel();
        var selectedRows = grid.selectionModel.getSelections();

        var fileRow = selectedRows[0];

        var downloadPath = fileRow.data.path;
        var downloader = new Ext.ux.file.Download({
            params: {
                method: 'Expressodriver.downloadFile',
                requestType: 'HTTP',
                id: '',
                path: downloadPath
            }
        }).start();
    },

    /**
     * is the download context menu option visible / enabled
     *
     * @param action
     * @param grants
     * @param records
     */
    isDownloadEnabled: function(action, grants, records) {
        for(var i=0; i<records.length; i++) {
            if(records[i].data.type === 'folder') {
                action.hide();
                return;
            }
        }
        action.show();

        var grid = this.scope.app.getMainScreen().getCenterPanel();
        var selectedRows = grid.selectionModel.getSelections();

        if(selectedRows.length > 1) {
            action.setDisabled(true);
        }
        else {
            action.setDisabled(false);
        }

    },

    /**
     * on pause
     * @param {} button
     * @param {} event
     */
    onPause: function (button, event) {

        var grid = this.scope;
        var gridStore = grid.store;
        gridStore.suspendEvents();
        var selectedRows = grid.selectionModel.getSelections();
        for(var i=0; i < selectedRows.length; i++) {
            var fileRecord = selectedRows[i];
            if(fileRecord.fileRecord) {
                fileRecord = fileRecord.fileRecord;
            }
            var upload = Tine.Tinebase.uploadManager.getUpload(fileRecord.get('uploadKey'));
            upload.setPaused(true);
        }
        gridStore.resumeEvents();
        grid.actionUpdater.updateActions(gridStore);
        this.scope.selectionModel.deselectRange(0, this.scope.selectionModel.getCount());
    },


    /**
     * on resume
     * @param {} button
     * @param {} event
     */
    onResume: function (button, event) {

        var grid = this.scope;
        var gridStore = grid.store;
        gridStore.suspendEvents();
        var selectedRows = grid.selectionModel.getSelections();
        for(var i=0; i < selectedRows.length; i++) {
            var fileRecord = selectedRows[i];
            if(fileRecord.fileRecord) {
                fileRecord = fileRecord.fileRecord;
            }
            var upload = Tine.Tinebase.uploadManager.getUpload(fileRecord.get('uploadKey'));
            upload.resumeUpload();
        }
        gridStore.resumeEvents();
        grid.actionUpdater.updateActions(gridStore);
        this.scope.selectionModel.deselectRange(0, this.scope.selectionModel.getCount());

    },

    /**
     * checks whether resume button shuold be enabled or disabled
     *
     * @param action
     * @param grants
     * @param records
     */
    isResumeEnabled: function(action, grants, records) {

        for(var i=0; i<records.length; i++) {

            var record = records[i];
            if(record.fileRecord) {
                record = record.fileRecord;
            }

            if(record.get('type') == 'folder') {
                action.hide();
                return;
            }
        }

        for(var i=0; i < records.length; i++) {

            var record = records[i];
            if(record.fileRecord) {
                record = record.fileRecord;
            }
            if(!record.get('status') || (record.get('type') != 'folder' &&  record.get('status') != 'uploading'
                    &&  record.get('status') != 'paused' && record.get('status') != 'pending')) {
                action.hide();
                return;
            }
        }

        action.show();

        for(var i=0; i < records.length; i++) {

            var record = records[i];
            if(record.fileRecord) {
                record = record.fileRecord;
            }

            if(record.get('status')) {
                action.setDisabled(false);
            }
            else {
                action.setDisabled(true);
            }
            if(record.get('status') && record.get('status') != 'paused') {
                action.setDisabled(true);
            }

        }
    },

    /**
     * checks whether pause button shuold be enabled or disabled
     *
     * @param action
     * @param grants
     * @param records
     */
    isPauseEnabled: function(action, grants, records) {

        for(var i=0; i<records.length; i++) {

            var record = records[i];
            if(record.fileRecord) {
                record = record.fileRecord;
            }

            if(record.get('type') === 'folder') {
                action.hide();
                return;
            }
        }

        for(var i=0; i < records.length; i++) {

            var record = records[i];
            if(record.fileRecord) {
                record = record.fileRecord;
            }

            if(!record.get('status') || (record.get('type ') != 'folder' && record.get('status') != 'paused'
                    &&  record.get('status') != 'uploading' && record.get('status') != 'pending')) {
                action.hide();
                return;
            }
        }

        action.show();

        for(var i=0; i < records.length; i++) {

            var record = records[i];
            if(record.fileRecord) {
                record = record.fileRecord;
            }

            if(record.get('status')) {
                action.setDisabled(false);
            }
            else {
                action.setDisabled(true);
            }
            if(record.get('status') && record.get('status') !=='uploading'){
                action.setDisabled(true);
            }

        }
    }
};

// extends Tine.widgets.tree.ContextMenu
Ext.applyIf(Tine.Expressodriver.GridContextMenu, Tine.widgets.tree.ContextMenu);
/*
 * Tine 2.0
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 */
Ext.ns('Tine.Expressodriver.Model');

/**
 * @namespace   Tine.Expressodriver.Model
 * @class       Tine.Expressodriver.Model.Node
 * @extends     Tine.Tinebase.data.Record
 * Node record definition
 *
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 */
Tine.Expressodriver.Model.Node = Tine.Tinebase.data.Record.create(Tine.Tinebase.Model.modlogFields.concat([
    { name: 'id' },
    { name: 'name' },
    { name: 'path' },
    { name: 'size' },
    { name: 'revision' },
    { name: 'type' },
    { name: 'contenttype' },
    { name: 'description' },
    { name: 'account_grants' },
    { name: 'description' },
    { name: 'object_id'},

    { name: 'relations' },
    { name: 'customfields' },
    { name: 'notes' },
    { name: 'tags' }
]), {
    appName: 'Expressodriver',
    modelName: 'Node',
    idProperty: 'id',
    titleProperty: 'name',
    recordName: 'File',
    recordsName: 'Files',
    containerName: 'Folder',
    containersName: 'Folders',

    /**
     * checks whether creating folders is allowed
     */
    isCreateFolderAllowed: function() {
        var grants = this.get('account_grants');

        if(!grants) {
            return false;
        }

        return this.get('type') == 'file' ? grants.editGrant : grants.addGrant;
    },

    isDropFilesAllowed: function() {
        var grants = this.get('account_grants');
        if(!grants) {
            return false;
        }
        else if(!grants.addGrant) {
            return false;
        }
        return true;
    },

    isDragable: function() {
        var grants = this.get('account_grants');

        if(!grants) {
            return false;
        }

        return true;
    }
});

/**
 * create Node from File
 *
 * @param {File} file
 */
Tine.Expressodriver.Model.Node.createFromFile = function(file) {
    return new Tine.Expressodriver.Model.Node({
        name: file.name ? file.name : file.fileName,  // safari and chrome use the non std. fileX props
        size: file.size || 0,
        type: 'file',
        contenttype: file.type ? file.type : file.fileType, // missing if safari and chrome
        revision: 0
    });
};

/**
 * default Expressodriver backend
 */
Tine.Expressodriver.fileRecordBackend =  new Tine.Tinebase.data.RecordProxy({
    appName: 'Expressodriver',
    modelName: 'Node',
    recordClass: Tine.Expressodriver.Model.Node,

    /**
     * creating folder
     *
     * @param name      folder name
     * @param options   additional options
     * @returns
     */
    createFolder: function(name, options) {

        options = options || {};
        var params = {
                application : this.appName,
                filename : name,
                type : 'folder',
                method : this.appName + ".createNode"
        };

        options.params = params;

        options.beforeSuccess = function(response) {
            var folder = this.recordReader(response);
            folder.set('client_access_time', new Date());
            return [folder];
        };

        options.success = function(result){
            var app = Tine.Tinebase.appMgr.get(Tine.Expressodriver.fileRecordBackend.appName);
            var grid = app.getMainScreen().getCenterPanel();
            var nodeData = Ext.util.JSON.decode(result);
            var newNode = app.getMainScreen().getWestPanel().getContainerTreePanel().createTreeNode(nodeData, parentNode);

            var parentNode = grid.currentFolderNode;
            if(parentNode) {
                parentNode.appendChild(newNode);
            }
            Ext.MessageBox.hide();
            grid.getStore().reload();
        };

        return this.doXHTTPRequest(options);
    },

    /**
     * is automatically called in generic GridPanel
     */
    saveRecord : function(record, request) {
        if(record.hasOwnProperty('fileRecord')) {
            return;
        } else {
            Tine.Tinebase.data.RecordProxy.prototype.saveRecord.call(this, record, request);
        }
    },

    /**
     * deleting file or folder
     *
     * @param items     files/folders to delete
     * @param options   additional options
     * @returns
     */
    deleteItems: function(items, options) {
        options = options || {};

        var filenames = new Array();
        var nodeCount = items.length;
        for(var i=0; i<nodeCount; i++) {
            filenames.push(items[i].data.path );
        }

        var params = {
            application: this.appName,
            filenames: filenames,
            method: this.appName + ".deleteNodes",
            timeout: 300000 // 5 minutes
        };

        options.params = params;

        options.beforeSuccess = function(response) {
            var folder = this.recordReader(response);
            folder.set('client_access_time', new Date());
            return [folder];
        };

        options.success = (function(result){
            var app = Tine.Tinebase.appMgr.get(Tine.Expressodriver.fileRecordBackend.appName),
                grid = app.getMainScreen().getCenterPanel(),
                treePanel = app.getMainScreen().getWestPanel().getContainerTreePanel(),
                nodeData = this.items;

            for(var i=0; i<nodeData.length; i++) {
                var treeNode = treePanel.getNodeById(nodeData[i].id);
                if(treeNode) {
                    treeNode.parentNode.removeChild(treeNode);
                }
            }

            grid.getStore().remove(nodeData);
            grid.selectionModel.deselectRange(0, grid.getStore().getCount());
            grid.pagingToolbar.refresh.enable();
            Ext.MessageBox.hide();

        }).createDelegate({items: items});
        var app = Tine.Tinebase.appMgr.get(this.appName);
        Ext.MessageBox.wait(_('Please wait'), app.i18n._('Deleting nodes...' ));
        return this.doXHTTPRequest(options);
    },

    /**
     * copy/move folder/files to a folder
     *
     * @param items files/folders to copy
     * @param targetPath
     * @param move
     */

    copyNodes : function(items, target, move, params) {

        var containsFolder = false,
            message = '',
            app = Tine.Tinebase.appMgr.get(Tine.Expressodriver.fileRecordBackend.appName);


        if(!params) {

            if(!target || !items || items.length < 1) {
                return false;
            }

            var sourceFilenames = new Array(),
            destinationFilenames = new Array(),
            forceOverwrite = false,
            treeIsTarget = false,
            treeIsSource = false,
            targetPath;

            if(target.data) {
                targetPath = target.data.path;
            }
            else {
                targetPath = target.attributes.path;
                treeIsTarget = true;
            }

            for(var i=0; i<items.length; i++) {

                var item = items[i];
                var itemData = item.data;
                if(!itemData) {
                    itemData = item.attributes;
                    treeIsSource = true;
                }
                sourceFilenames.push(itemData.path);

                var itemName = itemData.name;
                if(typeof itemName == 'object') {
                    itemName = itemName.name;
                }

                destinationFilenames.push(targetPath + '/' + itemName);
                if(itemData.type == 'folder') {
                    containsFolder = true;
                }
            };

            var method = "Expressodriver.copyNodes",
                message = app.i18n._('Copying data .. {0}');
            if(move) {
                method = "Expressodriver.moveNodes";
                message = app.i18n._('Moving data .. {0}');
            }

            params = {
                    application: this.appName,
                    sourceFilenames: sourceFilenames,
                    destinationFilenames: destinationFilenames,
                    forceOverwrite: forceOverwrite,
                    method: method
            };

        }
        else {
            message = app.i18n._('Copying data .. {0}');
            if(params.method == 'Expressodriver.moveNodes') {
                message = app.i18n._('Moving data .. {0}');
            }
        }

        this.loadMask = new Ext.LoadMask(app.getMainScreen().getCenterPanel().getEl(), {msg: String.format(_('Please wait')) + '. ' + String.format(message, '' )});
        app.getMainScreen().getWestPanel().setDisabled(true);
        app.getMainScreen().getNorthPanel().setDisabled(true);
        this.loadMask.show();

        Ext.Ajax.request({
            params: params,
            timeout: 300000, // 5 minutes
            scope: this,
            success: function(result, request){

                this.loadMask.hide();
                app.getMainScreen().getWestPanel().setDisabled(false);
                app.getMainScreen().getNorthPanel().setDisabled(false);

                var nodeData = Ext.util.JSON.decode(result.responseText),
                    treePanel = app.getMainScreen().getWestPanel().getContainerTreePanel(),
                    grid = app.getMainScreen().getCenterPanel();

                // Tree refresh
                if(treeIsTarget) {

                    for(var i=0; i<items.length; i++) {

                        var nodeToCopy = items[i];

                        if(nodeToCopy.data && nodeToCopy.data.type !== 'folder') {
                            continue;
                        }

                        if(move) {
                            var copiedNode = treePanel.cloneTreeNode(nodeToCopy, target),
                                nodeToCopyId = nodeToCopy.id,
                                removeNode = treePanel.getNodeById(nodeToCopyId);

                            if(removeNode && removeNode.parentNode) {
                                removeNode.parentNode.removeChild(removeNode);
                            }

                            target.appendChild(copiedNode);
                            copiedNode.setId(nodeData[i].id);
                        }
                        else {
                            var copiedNode = treePanel.cloneTreeNode(nodeToCopy, target);
                            target.appendChild(copiedNode);
                            copiedNode.setId(nodeData[i].id);

                        }
                    }
                }

                // Grid refresh
                grid.getStore().reload();
            },
            failure: function(response, request) {
                var nodeData = Ext.util.JSON.decode(response.responseText),
                    request = Ext.util.JSON.decode(request.jsonData);

                this.loadMask.hide();
                app.getMainScreen().getWestPanel().setDisabled(false);
                app.getMainScreen().getNorthPanel().setDisabled(false);

                Tine.Expressodriver.fileRecordBackend.handleRequestException(nodeData.data, request);
            }
        });

    },

    /**
     * upload file
     *
     * @param {} params Request parameters
     * @param String uploadKey
     * @param Boolean addToGridStore
     */
    createNode: function(params, uploadKey, addToGridStore) {
        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            grid = app.getMainScreen().getCenterPanel(),
            gridStore = grid.getStore();

        params.application = 'Expressodriver';
        params.method = 'Expressodriver.createNode';
        params.uploadKey = uploadKey;
        params.addToGridStore = addToGridStore;

        var onSuccess = (function(result, request){

            var nodeData = Ext.util.JSON.decode(response.responseText),
                fileRecord = Tine.Tinebase.uploadManager.upload(this.uploadKey);

            if(addToGridStore) {
                var recordToRemove = gridStore.query('name', fileRecord.get('name'));
                if(recordToRemove.items[0]) {
                    gridStore.remove(recordToRemove.items[0]);
                }

                fileRecord = Tine.Expressodriver.fileRecordBackend.updateNodeRecord(nodeData[i], fileRecord);
                var nodeRecord = new Tine.Expressodriver.Model.Node(nodeData[i]);

                nodeRecord.fileRecord = fileRecord;
                gridStore.add(nodeRecord);

            }
        }).createDelegate({uploadKey: uploadKey, addToGridStore: addToGridStore});

        var onFailure = (function(response, request) {

            var nodeData = Ext.util.JSON.decode(response.responseText),
                request = Ext.util.JSON.decode(request.jsonData);

            nodeData.data.uploadKey = this.uploadKey;
            nodeData.data.addToGridStore = this.addToGridStore;
            Tine.Expressodriver.fileRecordBackend.handleRequestException(nodeData.data, request);

        }).createDelegate({uploadKey: uploadKey, addToGridStore: addToGridStore});

        Ext.Ajax.request({
            params: params,
            timeout: 300000, // 5 minutes
            scope: this,
            success: onSuccess || Ext.emptyFn,
            failure: onFailure || Ext.emptyFn
        });
    },

    /**
     * upload files
     *
     * @param {} params Request parameters
     * @param [] uploadKeyArray
     * @param Boolean addToGridStore
     */
    createNodes: function(params, uploadKeyArray, addToGridStore) {
        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            grid = app.getMainScreen().getCenterPanel(),
            gridStore = grid.store;

        params.application = 'Expressodriver';
        params.method = 'Expressodriver.createNodes';
        params.uploadKeyArray = uploadKeyArray;
        params.addToGridStore = addToGridStore;


        var onSuccess = (function(response, request){

            var nodeData = Ext.util.JSON.decode(response.responseText);

            for(var i=0; i<this.uploadKeyArray.length; i++) {
                var fileRecord = Tine.Tinebase.uploadManager.upload(this.uploadKeyArray[i]);

                if(addToGridStore) {
                    fileRecord = Tine.Expressodriver.fileRecordBackend.updateNodeRecord(nodeData[i], fileRecord);
                    var nodeRecord = new Tine.Expressodriver.Model.Node(nodeData[i]);

                    nodeRecord.fileRecord = fileRecord;

                    var existingRecordIdx = gridStore.find('name', fileRecord.get('name'));
                    if(existingRecordIdx > -1) {
                        gridStore.removeAt(existingRecordIdx);
                        gridStore.insert(existingRecordIdx, nodeRecord);
                    } else {
                        gridStore.add(nodeRecord);
                    }
                }
            }

        }).createDelegate({uploadKeyArray: uploadKeyArray, addToGridStore: addToGridStore});

        var onFailure = (function(response, request) {

            var nodeData = Ext.util.JSON.decode(response.responseText),
                request = Ext.util.JSON.decode(request.jsonData);

            nodeData.data.uploadKeyArray = this.uploadKeyArray;
            nodeData.data.addToGridStore = this.addToGridStore;
            Tine.Expressodriver.fileRecordBackend.handleRequestException(nodeData.data, request);

        }).createDelegate({uploadKeyArray: uploadKeyArray, addToGridStore: addToGridStore});

        Ext.Ajax.request({
            params: params,
            timeout: 300000, // 5 minutes
            scope: this,
            success: onSuccess || Ext.emptyFn,
            failure: onFailure || Ext.emptyFn
        });


    },

    /**
     * exception handler for this proxy
     *
     * @param {Tine.Exception} exception
     */
    handleRequestException: function(exception, request) {
        Tine.Expressodriver.handleRequestException(exception, request);
    },

    /**
     * updates given record with nodeData from from response
     */
    updateNodeRecord : function(nodeData, nodeRecord) {

        for(var field in nodeData) {
            nodeRecord.set(field, nodeData[field]);
        };

        return nodeRecord;
    }


});


/**
 * get filtermodel of contact model
 *
 * @namespace Tine.Expressodriver.Model
 * @static
 * @return {Object} filterModel definition
 */
Tine.Expressodriver.Model.Node.getFilterModel = function() {
    var app = Tine.Tinebase.appMgr.get('Expressodriver');

    return [
        {label : _('Quick Search'), field : 'query', operators : [ 'contains' ]},
        {filtertype : 'tine.expressodriver.pathfiltermodel', app : app}
    ];
};

/**
 *  create model from settinf of Expressodriver
 */
Tine.Expressodriver.Model.Settings = Tine.Tinebase.data.Record.create([
        {name: 'id'},
        {name: 'default'},
        {name: 'adapters'}
    ], {
    appName: 'Expressodriver',
    modelName: 'Settings',
    idProperty: 'id',
    titleProperty: 'title',
    recordName: 'Settings',
    recordsName: 'Settingss',
    containerName: 'Settings',
    containersName: 'Settings',
    getTitle: function() {
        return this.recordName;
    }
});

/**
 * set settings backend
 */
Tine.Expressodriver.settingsBackend = new Tine.Tinebase.data.RecordProxy({
    appName: 'Expressodriver',
    modelName: 'Settings',
    recordClass: Tine.Expressodriver.Model.Settings
});

/**
 * get randon id
 * @param store
 * @returns {Integer|Number}
 */
Tine.Expressodriver.Model.getRandomUnusedId = function(store) {
    var result;
    do {
        result = Tine.Tinebase.common.getRandomNumber(0, 21474836);
    } while (store.getById(result) != undefined)

    return result;
};/*
 * Tine 2.0
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 *
 */
Ext.ns('Tine.Expressodriver');

/**
 * @namespace   Tine.widgets.container
 * @class       Tine.Expressodriver.PathFilterModel
 * @extends     Tine.widgets.grid.FilterModel
 *
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 *
 * @TODO make valueRenderer a path picker widget
 */
Tine.Expressodriver.PathFilterModel = Ext.extend(Tine.widgets.grid.FilterModel, {
    /**
     * @cfg {Tine.Tinebase.Application} app
     */
    app: null,

    /**
     * @cfg {Array} operators allowed operators
     */
    operators: ['equals'],

    /**
     * @cfg {String} field path
     */
    field: 'path',

    /**
     * @cfg {String} defaultOperator default operator, one of <tt>{@link #operators} (defaults to equals)
     */
    defaultOperator: 'equals',

    /**
     * @cfg {String} defaultValue default value (defaults to all)
     */
    defaultValue: '/',

    /**
     * @private
     */
    initComponent: function() {
        this.label = this.app.i18n._('path');

        Tine.Expressodriver.PathFilterModel.superclass.initComponent.call(this);
    },

    /**
     * value renderer
     *
     * @param {Ext.data.Record} filter line
     * @param {Ext.Element} element to render to
     */
    valueRenderer: function(filter, el) {

        var filterValue = this.defaultValue;
        if(filter.data.value) {
            if(typeof filter.data.value == 'object') {
                filterValue = filter.data.value.path;
            } else {
                filterValue = filter.data.value;
            }
        }

        var value = new Ext.ux.form.ClearableTextField({
            filter: filter,
            width: this.filterValueWidth,
            id: 'tw-ftb-frow-valuefield-' + filter.id,
            renderTo: el,
            value: filterValue,
            emptyText: this.emptyText
        });

        value.on('specialkey', function(field, e){
            if(e.getKey() == e.ENTER){
                this.onFiltertrigger();
            }
        }, this);

        value.origSetValue = value.setValue.createDelegate(value);
        value.setValue = function(value) {
            if (value && value.path) {
                value = value.path;
            }
            else if(Ext.isString(value) && (!value.charAt(0) || value.charAt(0) != '/')) {
                value = '/' + value;
            }

            return this.origSetValue(value);
        };

        return value;
    }
});

Tine.widgets.grid.FilterToolbar.FILTERS['tine.expressodriver.pathfiltermodel'] = Tine.Expressodriver.PathFilterModel;
/*
 * Tine 2.0
 * Expressodriver combo box and store
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 *
 */

Ext.ns('Tine.Expressodriver');

/**
 * Node selection combo box
 *
 * @namespace   Tine.Expressodriver
 * @class       Tine.Expressodriver.SearchCombo
 * @extends     Ext.form.ComboBox
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 *
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressodriver.SearchCombo
 */
Tine.Expressodriver.SearchCombo = Ext.extend(Tine.Tinebase.widgets.form.RecordPickerComboBox, {

    allowBlank: false,
    itemSelector: 'div.search-item',
    minListWidth: 200,

    /**
     * init component
     * @private
     */
    initComponent: function(){
        this.recordClass = Tine.Expressodriver.Model.Node;
        this.recordProxy = Tine.Expressodriver.recordBackend;
        this.additionalFilters = [
            {field: 'recursive', operator: 'equals', value: true },
            {field: 'path', operator: 'equals', value: '/' }
        ];
        this.initTemplate();
        Tine.Expressodriver.SearchCombo.superclass.initComponent.call(this);
    },

    /**
     * init template
     * @private
     */
    initTemplate: function() {
        // Custom rendering Template
        // TODO move style def to css ?
        if (! this.tpl) {
            this.tpl = new Ext.XTemplate(
                '<tpl for="."><div class="search-item">',
                    '<table cellspacing="0" cellpadding="2" border="0" style="font-size: 11px;" width="100%">',
                        '<tr>',
                            '<td ext:qtip="{[this.renderPathName(values)]}" style="height:16px">{[this.renderFileName(values)]}</td>',
                        '</tr>',
                    '</table>',
                '</div></tpl>',
                {
                    renderFileName: function(values) {
                        return Ext.util.Format.htmlEncode(values.name);
                    },
                    renderPathName: function(values) {
                        return Ext.util.Format.htmlEncode(values.path.replace(values.name, ''));
                    }

                }
            );
        }
    },

    getValue: function() {
            return Tine.Expressodriver.SearchCombo.superclass.getValue.call(this);
    },

    setValue: function (value) {
        return Tine.Expressodriver.SearchCombo.superclass.setValue.call(this, value);
    }

});
/**
 * register search combo
 */
Tine.widgets.form.RecordPickerManager.register('Expressodriver', 'Node', Tine.Expressodriver.SearchCombo);
/*
 * Tine 2.0
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 *
 */
Ext.ns('Tine.Expressodriver');

/**
 * filter plugin for container tree
 *
 * @namespace Tine.widgets.tree
 * @class     Tine.Expressodriver.PathFilterPlugin
 * @extends   Tine.widgets.grid.FilterPlugin
 */
Tine.Expressodriver.PathFilterPlugin = Ext.extend(Tine.widgets.tree.FilterPlugin, {

    /**
     * select tree node(s)
     *
     * @param {String} value
     */
    selectValue: function(value) {
        var values = Ext.isArray(value) ? value : [value];
        Ext.each(values, function(value) {
            var path = Ext.isString(value) ? value : (value ? value.path : '') || '/',
                treePath = this.treePanel.getTreePath(path),
                attr = null;

            if (treePath.split('/').length === 3){
                attr = 'name';
            }
            this.selectPath.call(this.treePanel, treePath, attr, function() {
                // mark this expansion as done and check if all are done
                value.isExpanded = true;
                var allValuesExpanded = true;
                Ext.each(values, function(v) {
                    allValuesExpanded &= v.isExpanded;
                }, this);

                if (allValuesExpanded) {
                    this.treePanel.getSelectionModel().resumeEvents();

                    // @TODO remove this code when fm is cleaned up conceptually
                    //       currentFolderNode -> currentFolder
                    this.treePanel.updateActions(this.treePanel.getSelectionModel(), this.treePanel.getSelectionModel().getSelectedNode());
                    Tine.Tinebase.appMgr.get('Expressodriver').getMainScreen().getCenterPanel().currentFolderNode = this.treePanel.getSelectionModel().getSelectedNode();
                }
            }.createDelegate(this), true);
        }, this);
    }
});
/*
 * Tine 2.0
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 */
Ext.ns('Tine.Expressodriver');

/**
 * @namespace   Tine.Expressodriver
 * @class       Tine.Expressodriver.NodeEditDialog
 * @extends     Tine.widgets.dialog.EditDialog
 *
 * <p>Node Compose Dialog</p>
 * <p></p>
 *
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 *
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressodriver.NodeEditDialog
 */
Tine.Expressodriver.NodeEditDialog = Ext.extend(Tine.widgets.dialog.EditDialog, {

    /**
     * @private
     */
    windowNamePrefix: 'NodeEditWindow_',
    appName: 'Expressodriver',
    recordClass: Tine.Expressodriver.Model.Node,
    recordProxy: Tine.Expressodriver.fileRecordBackend,
    tbarItems: null,
    evalGrants: true,
    showContainerSelector: false,

    initComponent: function() {
        this.app = Tine.Tinebase.appMgr.get('Expressodriver');
        this.downloadAction = new Ext.Action({
            requiredGrant: 'readGrant',
            allowMultiple: false,
            actionType: 'download',
            text: this.app.i18n._('Save locally'),
            handler: this.onDownload,
            iconCls: 'action_expressodriver_save_all',
            disabled: false,
            scope: this
        });

        this.tbarItems = [this.downloadAction];

        Tine.Expressodriver.NodeEditDialog.superclass.initComponent.call(this);

        this.action_saveAndClose.setDisabled(true);

    },

    /**
     * download file
     */
    onDownload: function() {
        var downloader = new Ext.ux.file.Download({
            params: {
                method: 'Expressodriver.downloadFile',
                requestType: 'HTTP',
                path: '',
                id: this.record.get('id')
            }
        }).start();
    },

    /**
     * returns dialog
     * @return {Object}
     * @private
     */
    getFormItems: function() {
        var formFieldDefaults = {
            xtype:'textfield',
            anchor: '100%',
            labelSeparator: '',
            columnWidth: .5,
            readOnly: true,
            disabled: true
        };

        return {
            xtype: 'tabpanel',
            border: false,
            plain:true,
            plugins: [{
                ptype : 'ux.tabpanelkeyplugin'
            }],
            activeTab: 0,
            border: false,
            items:[{
                title: this.app.i18n._('Node'),
                autoScroll: true,
                border: false,
                frame: true,
                layout: 'border',
                items: [{
                    region: 'center',
                    layout: 'hfit',
                    border: false,
                    items: [{
                        xtype: 'fieldset',
                        layout: 'hfit',
                        autoHeight: true,
                        title: this.app.i18n._('Node'),
                        items: [{
                            xtype: 'columnform',
                            labelAlign: 'top',
                            formDefaults: formFieldDefaults,
                            items: [[{
                                    fieldLabel: this.app.i18n._('Name'),
                                    name: 'name',
                                    allowBlank: false,
                                    readOnly: true,
                                    columnWidth: .75,
                                    disabled: false
                                }, {
                                    fieldLabel: this.app.i18n._('Type'),
                                    name: 'contenttype',
                                    columnWidth: .25
                                }],[
                                Tine.widgets.form.RecordPickerManager.get('Addressbook', 'Contact', {
                                    userOnly: true,
                                    useAccountRecord: true,
                                    blurOnSelect: true,
                                    fieldLabel: this.app.i18n._('Modified By'),
                                    name: 'last_modified_by'
                                }), {
                                    fieldLabel: this.app.i18n._('Last Modified'),
                                    name: 'last_modified_time',
                                    xtype: 'datefield'
                                }
                                ]]
                        }]
                    }]
                }]
            }]
        };
    },

    /**
     * creates the relations panel, if relations are defined
     */
    initRelationsPanel: function() {
        // do not initialize relations
    }
});

/**
 * Expressodriver Edit Popup
 *
 * @param   {Object} config
 * @return  {Ext.ux.Window}
 */
Tine.Expressodriver.NodeEditDialog.openWindow = function (config) {
    var id = (config.record && config.record.id) ? config.record.id : 0;
    var window = Tine.WindowFactory.getWindow({
        width: 570,
        height: 270,
        name: Tine.Expressodriver.NodeEditDialog.prototype.windowNamePrefix + id,
        contentPanelConstructor: 'Tine.Expressodriver.NodeEditDialog',
        contentPanelConstructorConfig: config
    });

    return window;
};
/*
 * Tine 2.0
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 */

Ext.ns('Tine.Expressodriver');

/**
 * @namespace Tine.Expressodriver
 * @class Tine.Expressodriver.NodeTreePanel
 * @extends Tine.widgets.container.TreePanel
 *
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 */

Tine.Expressodriver.NodeTreePanel = function(config) {
    Ext.apply(this, config);

    this.addEvents(
        /**
         * @event containeradd
         * Fires when a folder was added
         * @param {folder} the new folder
         */
        'containeradd',
        /**
         * @event containerdelete
         * Fires when a folder got deleted
         * @param {folder} the deleted folder
         */
        'containerdelete',
        /**
         * @event containerrename
         * Fires when a folder got renamed
         * @param {folder} the renamed folder
         */
        'containerrename'
    );

    Tine.Expressodriver.NodeTreePanel.superclass.constructor.call(this);
};

/**
 * Extend node tree panel and tree panel *
 */
Ext.extend(Tine.Expressodriver.NodeTreePanel, Tine.widgets.container.TreePanel, {

    filterMode : 'filterToolbar',

    recordClass : Tine.Expressodriver.Model.Node,

    allowMultiSelection : false,

    defaultContainerPath: '/',

    ddGroup: 'fileDDGroup',

    enableDD: true,

    initComponent: function() {

        this.on('containeradd', this.onFolderAdd, this);
        this.on('containerrename', this.onFolderRename, this);
        this.on('containerdelete', this.onFolderDelete, this);
        this.on('nodedragover', this.onNodeDragOver, this);

        Tine.Tinebase.uploadManager.on('update', this.onUpdate);

        if (! this.appName && this.recordClass) {
            this.appName = this.recordClass.getMeta('appName');
        }
        if (! this.app) {
            this.app = Tine.Tinebase.appMgr.get(this.appName);
        }

        if (this.allowMultiSelection) {
            this.selModel = new Ext.tree.MultiSelectionModel({});
        }

        var containerName = this.recordClass ? this.recordClass.getContainerName() : 'container';
        var containersName = this.recordClass ? this.recordClass.getContainersName() : 'containers';

        this.containerName = this.containerName || this.app.i18n.n_hidden(containerName, containersName, 1);
        this.containersName = this.containersName || this.app.i18n._hidden(containersName);

        this.loader = this.loader || new Tine.widgets.tree.Loader({
            getParams: this.onBeforeLoad.createDelegate(this),
            inspectCreateNode: this.onBeforeCreateNode.createDelegate(this)
        });

        this.root = {
            path: '/',
            cls: 'tinebase-tree-hide-collapsetool',
            expanded: true,
            children: this.getExtraItems()
        };

        this.initContextMenu();

        this.getSelectionModel().on('beforeselect', this.onBeforeSelect, this);
        this.getSelectionModel().on('selectionchange', this.onSelectionChange, this);
        this.on('click', this.onClick, this);
        this.on('contextmenu', this.onContextMenu, this);
        this.on('beforenodedrop', this.onBeforeNodeDrop, this);
        this.on('beforecontainerrename', this.onBeforeFolderRename, this);
        this.on('append', this.onAppendNode, this);

        Tine.widgets.container.TreePanel.superclass.initComponent.call(this);

        // init drop zone
        this.dropConfig = {
            ddGroup: this.ddGroup || 'fileDDGroup',
            appendOnly: this.ddAppendOnly === true,
            /**
             * @todo check acl!
             */
            onNodeOver : function(n, dd, e, data) {

                var preventDrop = false,
                    selectionContainsFiles = false;

                if(dd.dragData.selections) {
                    for(var i=0; i<dd.dragData.selections.length; i++) {
                        if(n.node.id == dd.dragData.selections[i].id) {
                            preventDrop = true;
                        } else if (this.isParentPath(dd.dragData.selections[i].data.path, n.node.attributes.path)) {
                            preventDrop = true;
                        }
                        if(dd.dragData.selections[i].data.type == 'file') {
                            selectionContainsFiles = true;
                        }
                    }
                }
                else if(dd.dragData.node && dd.dragData.node.id == n.node.id) {
                    preventDrop = true;
                }

                if(selectionContainsFiles && !n.node.attributes.account_grants) {
                    preventDrop = true;
                }

                if(n.node.isAncestor(dd.dragData.node)) {
                    preventDrop = true;
                }

                return n.node.attributes.nodeRecord.isCreateFolderAllowed()
                    && (!dd.dragData.node || dd.dragData.node.attributes.nodeRecord.isDragable())
                    && !preventDrop ? 'x-dd-drop-ok' : false;
            },

            isValidDropPoint: function(n, op, dd, e){

                var preventDrop = false,
                selectionContainsFiles = false;

                if(dd.dragData.selections) {
                    for(var i=0; i<dd.dragData.selections.length; i++) {
                        if(n.node.id == dd.dragData.selections[i].id) {
                            preventDrop = true;
                        } else if (this.isParentPath(dd.dragData.selections[i].data.path, n.node.attributes.path)) {
                            preventDrop = true;
                        }
                        if(dd.dragData.selections[i].data.type == 'file') {
                            selectionContainsFiles = true;
                        }
                    }
                }
                else if(dd.dragData.node && dd.dragData.node.id == n.node.id) {
                    preventDrop = true;
                }

                if(selectionContainsFiles && !n.node.attributes.account_grants) {
                    preventDrop = true;
                }

                if(n.node.isAncestor(dd.dragData.node)) {
                    preventDrop = true;
                }

                return n.node.attributes.nodeRecord.isCreateFolderAllowed()
                        && (!dd.dragData.node || dd.dragData.node.attributes.nodeRecord.isDragable())
                        && !preventDrop;
            },
            completeDrop: function(de) {
                var ns = de.dropNode, p = de.point, t = de.target;
                t.ui.endDrop();
                this.tree.fireEvent("nodedrop", de);
            },

            /**
             * checks if the path needle is a sub path of haystack
             */
            isSubPath: function(haystack, needle) {
                var h = haystack.split('/');
                var n = needle.split('/');
                var res = true;

                for (var index = 0; index < h.length; index++) {

                    if (n.length <= index) {
                        break;
                    }

                    if (h[index] != n[index]) {
                        res = false;
                    }
                }

                return res;
            },

            /**
             * checks if the path needle is parent path of haystack node
             */
            isParentPath: function(haystack, needle) {
                return haystack === needle + '/' + haystack.match(/[^\/]*$/);
            }
        };

        this.dragConfig = {
            ddGroup: this.ddGroup || 'fileDDGroup',
            scroll: this.ddScroll,
            /**
             * tree node dragzone modified, dragged node doesn't get selected
             *
             * @param e
             */
            onInitDrag: function(e) {
                var data = this.dragData;
                this.tree.eventModel.disable();
                this.proxy.update("");
                data.node.ui.appendDDGhost(this.proxy.ghost.dom);
                this.tree.fireEvent("startdrag", this.tree, data.node, e);
            }
        };

        this.plugins = this.plugins || [];
        this.plugins.push({
            ptype : 'ux.browseplugin',
            enableFileDialog: false,
            multiple : true,
            handler : this.dropIntoTree
        });
    },

    /**
     * Tine.widgets.tree.FilterPlugin
     * returns a filter plugin to be used in a grid
     *
     * Tine.widgets.tree.FilterPlugin
     * Tine.Expressodriver.PathFilterPlugin
     */
    getFilterPlugin: function() {
        if (!this.filterPlugin) {
            this.filterPlugin = new Tine.Expressodriver.PathFilterPlugin({
                treePanel: this,
                field: 'path',
                nodeAttributeField: 'path'
            });
        }

        return this.filterPlugin;
    },

    /**
     * returns the personal root path
     * @returns {String}
     */
    getRootPath: function() {
        return '/';
    },

    /**
     * returns params for async request
     *
     * @param {Ext.tree.TreeNode} node
     * @return {Object}
     */
    onBeforeLoad: function(node) {
        var owner = true;
        var path = node.attributes.path;
        var newPath = path;
        var params = {
            method: 'Expressodriver.searchNodes',
            application: this.app.appName,
            owner: owner,
            filter: [
                     {field: 'path', operator:'equals', value: newPath},
                     {field: 'type', operator:'equals', value: 'folder'}
                     ],
            paging: {dir: 'ASC', limit: 50, sort: 'name', start: 0}
        };

        return params;
    },

    onBeforeCreateNode: function(attr) {
        Tine.Expressodriver.NodeTreePanel.superclass.onBeforeCreateNode.apply(this, arguments);

        attr.leaf = false;

        if(attr.name && typeof attr.name == 'object') {
            Ext.apply(attr, {
                text: Ext.util.Format.htmlEncode(attr.name.name),
                qtip: Tine.Tinebase.common.doubleEncode(attr.name.name)
            });
        }

        // copy 'real' data to a node record NOTE: not a full record as we have no record reader here
        var nodeData = Ext.copyTo({}, attr, Tine.Expressodriver.Model.Node.getFieldNames());
        attr.nodeRecord = new Tine.Expressodriver.Model.Node(nodeData);
    },

    /**
     * initiates tree context menues
     *
     * @private
     */
    initContextMenu: function() {

        this.contextMenuUserFolder = Tine.widgets.tree.ContextMenu.getMenu({
            nodeName: this.app.i18n._(this.containerName),
            actions: ['add', 'reload', 'delete', 'rename'],
            scope: this,
            backend: 'Expressodriver',
            backendModel: 'Node'
        });

        this.contextMenuRootFolder = Tine.widgets.tree.ContextMenu.getMenu({
            nodeName: this.app.i18n._(this.containerName),
            actions: ['add', 'reload'],
            scope: this,
            backend: 'Expressodriver',
            backendModel: 'Node'
        });

        this.contextMenuOtherUserFolder = Tine.widgets.tree.ContextMenu.getMenu({
            nodeName: this.app.i18n._(this.containerName),
            actions: ['reload'],
            scope: this,
            backend: 'Expressodriver',
            backendModel: 'Node'
        });

        this.contextMenuContainerFolder = Tine.widgets.tree.ContextMenu.getMenu({
            nodeName: this.app.i18n._(this.containerName),
            actions: ['add', 'reload', 'delete', 'rename', 'grants', 'properties'],
            scope: this,
            backend: 'Expressodriver',
            backendModel: 'Node'
        });

        this.contextMenuReloadFolder = Tine.widgets.tree.ContextMenu.getMenu({
            nodeName: this.app.i18n._(this.containerName),
            actions: ['reload', 'properties'],
            scope: this,
            backend: 'Expressodriver',
            backendModel: 'Node'
        });
    },

    /**
     * @private
     * - select default path
     */
    afterRender: function() {
        Tine.Expressodriver.NodeTreePanel.superclass.afterRender.call(this);
    },

    /**
     * show context menu
     *
     * @param {Ext.tree.TreeNode} node
     * @param {Ext.EventObject} event
     */
    onContextMenu: function(node, event) {
        this.ctxNode = node;
        var container = node.attributes.nodeRecord.data,
            path = container.path;

        if (! Ext.isString(path) || node.isRoot) {
            return;
        }

        Tine.log.debug('Tine.Expressodriver.NodeTreePanel::onContextMenu - context node:');
        Tine.log.debug(node);

        if (node.parentNode && node.parentNode.isRoot) {
            this.contextMenuRootFolder.showAt(event.getXY());
        } else {
            this.contextMenuUserFolder.showAt(event.getXY());
        }
    },

    /**
     * updates grid actions
     * @todo move to grid / actionUpdater
     *
     * @param {} sm     SelectionModel
     * @param {Ext.tree.TreeNode} node
     */
    updateActions: function(sm, node) {
        var grid = this.app.getMainScreen().getCenterPanel();

        grid.action_deleteRecord.disable();
        grid.action_upload.disable();

        if(!!node && !!node.isRoot) {
            grid.action_goUpFolder.disable();
        }
        else {
            grid.action_goUpFolder.enable();
        }

        if(node && node.attributes && node.attributes.nodeRecord.isCreateFolderAllowed()) {
            grid.action_createFolder.enable();
        }
        else {
            grid.action_createFolder.disable();
        }

        if(node && node.attributes && node.attributes.nodeRecord.isDropFilesAllowed()) {
            grid.action_upload.enable();
        }
        else {
            grid.action_upload.disable();
        }
    },

        /**
     * called when tree selection changes
     *
     * @param {} sm     SelectionModel
     * @param {Ext.tree.TreeNode} node
     */
    onSelectionChange: function(sm, node) {
        this.updateActions(sm, node);
        var grid = this.app.getMainScreen().getCenterPanel();

        grid.currentFolderNode = node;
        Tine.Expressodriver.NodeTreePanel.superclass.onSelectionChange.call(this, sm, node);

    },

    getExtraItems: function(){
    //    return [];
    },

    /**
     * convert filesystem path to treePath
     *
     * NOTE: only the first depth gets converted!
     *       fs pathes of not yet loaded tree nodes can't be converted!
     *
     * @param {String} containerPath
     * @return {String} tree path
     */
    getTreePath: function(path) {
        var treePath = '/' + this.getRootNode().id;

        if (path && path != '/') {
           treePath += String(path);
        }

        return treePath;
    },

    /**
     * Expands a specified path in this TreePanel. A path can be retrieved from a node with {@link Ext.data.Node#getPath}
     *
     * NOTE: path does not consist of id's starting from the second depth
     *
     * @param {String} path
     * @param {String} attr (optional) The attribute used in the path (see {@link Ext.data.Node#getPath} for more info)
     * @param {Function} callback (optional) The callback to call when the expand is complete. The callback will be called with
     * (bSuccess, oLastNode) where bSuccess is if the expand was successful and oLastNode is the last node that was expanded.
     */
    expandPath : function(path, attr, callback){
        var keys = path.split(this.pathSeparator);
        var curNode = this.root;
        var curPath = curNode.attributes.path;
        var index = 1;
        var f = function(){
            if(++index == keys.length){
                if(callback){
                    callback(true, curNode);
                }
                return;
            }

            if (index > 2) {
                var c = curNode.findChild('path', curPath + '/' + keys[index]);
            } else {
                var c = curNode.findChild('name', keys[index]);
            }
            if(!c){
                if(callback){
                    callback(false, curNode);
                }
                return;
            }
            curNode = c;
            curPath = c.attributes.path;
            c.expand(false, false, f);
        };
        curNode.expand(false, false, f);
    },

    /**
     * files/folder got dropped on node
     *
     * @param {Object} dropEvent
     * @private
     */
    onBeforeNodeDrop: function(dropEvent) {
        var nodes, target = dropEvent.target;

        if(dropEvent.data.selections) {
            nodes = dropEvent.data.grid.selModel.selections.items;
        }

        if(!nodes && dropEvent.data.node) {
            nodes = [dropEvent.data.node];
        }

        Tine.Expressodriver.fileRecordBackend.copyNodes(nodes, target, !dropEvent.rawEvent.ctrlKey);

        dropEvent.dropStatus = true;
        return true;
    },

    /**
     * folder delete handler
     */
    onFolderDelete: function(node) {
        var grid = this.app.getMainScreen().getCenterPanel();
        if(grid.currentFolderNode.isAncestor && typeof grid.currentFolderNode.isAncestor == 'function'
            && grid.currentFolderNode.isAncestor(node)) {
            node.parentNode.select();
        }
        grid.getStore().reload();
    },

    /**
     * clone a tree node / create a node from grid node
     *
     * @param node
     * @returns {Ext.tree.AsyncTreeNode}
     */
    cloneTreeNode: function(node, target) {
        var targetPath = target.attributes.path,
            newPath = '',
            copy;

        if(node.attributes) {
            var nodeName = node.attributes.name;
            if(typeof nodeName == 'object') {
                nodeName = nodeName.name;
            }
            newPath = targetPath + '/' + nodeName;

            copy = new Ext.tree.AsyncTreeNode({text: node.text, path: newPath, name: node.attributes.name
                , nodeRecord: node.attributes.nodeRecord, account_grants: node.attributes.account_grants});
        }
        else {
            var nodeName = node.data.name;
            if(typeof nodeName == 'object') {
                nodeName = nodeName.name;
            }

            var nodeData = Ext.copyTo({}, node.data, Tine.Expressodriver.Model.Node.getFieldNames());
            var newNodeRecord = new Tine.Expressodriver.Model.Node(nodeData);

            newPath = targetPath + '/' + nodeName;
            copy = new Ext.tree.AsyncTreeNode({text: nodeName, path: newPath, name: node.data.name
                , nodeRecord: newNodeRecord, account_grants: node.data.account_grants});
        }

        copy.attributes.nodeRecord.beginEdit();
        copy.attributes.nodeRecord.set('path', newPath);
        copy.attributes.nodeRecord.endEdit();

        copy.parentNode = target;
        return copy;
    },

    /**
     * create Tree node by given node data
     *
     * @param nodeData
     * @param target
     * @returns {Ext.tree.AsyncTreeNode}
     */
    createTreeNode: function(nodeData, target) {
        var nodeName = nodeData.name;
        if(typeof nodeName == 'object') {
            nodeName = nodeName.name;
        }

        var newNodeRecord = new Tine.Expressodriver.Model.Node(nodeData);

        var newNode = new Ext.tree.AsyncTreeNode({
            text: nodeName,
            path: nodeData.path,
            name: nodeData.name,
            nodeRecord: newNodeRecord,
            account_grants: nodeData.account_grants,
            id: nodeData.id
        })

        newNode.attributes.nodeRecord.beginEdit();
        newNode.attributes.nodeRecord.set('path', nodeData.path);
        newNode.attributes.nodeRecord.endEdit();

        newNode.parentNode = target;
        return newNode;

    },

    /**
     * TODO: move to Upload class or elsewhere??
     * updating fileRecord after creating node
     *
     * @param response
     * @param request
     * @param upload
     */
    onNodeCreated: function(response, request, upload) {

        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
        grid = app.getMainScreen().getCenterPanel();

        var record = Ext.util.JSON.decode(response.responseText);

        var fileRecord = upload.fileRecord;
        fileRecord.beginEdit();
        fileRecord.set('contenttype', record.contenttype);
        fileRecord.set('created_by', Tine.Tinebase.registry.get('currentAccount'));
        fileRecord.set('creation_time', record.creation_time);
        fileRecord.set('revision', record.revision);
        fileRecord.set('last_modified_by', record.last_modified_by);
        fileRecord.set('last_modified_time', record.last_modified_time);
        fileRecord.set('status', 'complete');
        fileRecord.set('progress', 100);
        fileRecord.set('name', record.name);
        fileRecord.set('path', record.path);
        fileRecord.commit(false);

        upload.fireEvent('update', 'uploadfinished', upload, fileRecord);

        grid.pagingToolbar.refresh.enable();

    },

    /**
     * copies uploaded temporary file to target location
     *
     * @param upload    {Ext.ux.file.Upload}
     * @param file  {Ext.ux.file.Upload.file}
     */
    onUploadComplete: function(upload, file) {
        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            treePanel = app.getMainScreen().getWestPanel().getContainerTreePanel();

     // check if we are responsible for the upload
        if (upload.fmDirector != treePanel) return;

        // $filename, $type, $tempFileId, $forceOverwrite
        Ext.Ajax.request({
            timeout: 10*60*1000, // Overriding Ajax timeout - important!
            params: {
                method: 'Expressodriver.createNode',
                filename: upload.id,
                type: 'file',
                tempFileId: file.get('id'),
                forceOverwrite: true
            },
            success: treePanel.onNodeCreated.createDelegate(this, [upload], true),
            failure: treePanel.onNodeCreated.createDelegate(this, [upload], true)
        });

    },

    /**
     * on upload failure
     *
     * @private
     */
    onUploadFail: function () {
        Ext.MessageBox.alert(
            _('Upload Failed'),
            _('Could not upload file. Filesize could be too big. Please notify your Administrator. Max upload size:') + ' ' + Tine.Tinebase.registry.get('maxFileUploadSize')
        ).setIcon(Ext.MessageBox.ERROR);
    },

    /**
     * add folder handler
     */
    onFolderAdd: function(nodeData) {

        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            grid = app.getMainScreen().getCenterPanel();

        grid.getStore().reload();
        if(nodeData.error) {
            Tine.log.debug(nodeData);
        }
    },

    /**
     * handles before tree node / aka folder renaming
     */
    onBeforeFolderRename: function(node) {
        var app = Tine.Tinebase.appMgr.get('Expressodriver');
        Ext.MessageBox.wait(_('Please wait'), app.i18n._('Renaming nodes...' ));
    },

    /**
     * handles renaming of a tree node / aka folder
     */
    onFolderRename: function(nodeData, node, newName) {
        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            grid = app.getMainScreen().getCenterPanel();

        if(nodeData[0]) {
            nodeData = nodeData[0];
        };

        node.attributes.nodeRecord.beginEdit();
        node.attributes.nodeRecord.set('name', newName);
        node.attributes.nodeRecord.set('path', nodeData.path);
        node.attributes.path = nodeData.path;
        node.attributes.nodeRecord.commit(false);

        if(typeof node.attributes.name == 'object') {
            node.attributes.name.name = newName;
        }
        else {
            node.attributes.name = newName;
        }

        grid.currenFolderNode = node;

        Ext.MessageBox.hide();
        Tine.Expressodriver.NodeTreePanel.superclass.onSelectionChange.call(this, this.getSelectionModel(), node);

    },

    /**
     * upload update handler
     *
     * @param change {String} kind of change
     * @param upload {Ext.ux.file.Upload} upload
     * @param fileRecord {file} fileRecord
     *
     */
    onUpdate: function(change, upload, fileRecord) {

        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            grid = app.getMainScreen().getCenterPanel(),
            treePanel = app.getMainScreen().getWestPanel().getContainerTreePanel(),
            rowsToUpdate = grid.getStore().query('name', fileRecord.get('name'));

        if(change == 'uploadstart') {
            Tine.Tinebase.uploadManager.onUploadStart();
        }
        else if(change == 'uploadfailure') {
            treePanel.onUploadFail();
        }

        if(rowsToUpdate.get(0)) {
            if(change == 'uploadcomplete') {
                treePanel.onUploadComplete(upload, fileRecord);
            }
            else if(change == 'uploadfinished') {
                rowsToUpdate.get(0).set('size', upload.fileSize);
                rowsToUpdate.get(0).set('contenttype', fileRecord.get('contenttype'));
            }
            rowsToUpdate.get(0).afterEdit();
            rowsToUpdate.get(0).commit(false);
        }
    },

    /**
     * handels tree drop of object from outside the browser
     *
     * @param fileSelector
     * @param targetNodeId
     */
    dropIntoTree: function(fileSelector, event) {

        var treePanel = fileSelector.component,
            app = treePanel.app,
            grid = app.getMainScreen().getCenterPanel(),
            targetNode,
            targetNodePath;


        var targetNodeId;
        var treeNodeAttribute = event.getTarget('div').attributes['ext:tree-node-id'];
        if(treeNodeAttribute) {
            targetNodeId = treeNodeAttribute.nodeValue;
            targetNode = treePanel.getNodeById(targetNodeId);
            targetNodePath = targetNode.attributes.path;

        };

        if(!targetNode.attributes.nodeRecord.isDropFilesAllowed()) {
            Ext.MessageBox.alert(
                    _('Upload Failed'),
                    app.i18n._('Putting files in this folder is not allowed!')
                ).setIcon(Ext.MessageBox.ERROR);

            return;
        };

        var files = fileSelector.getFileList(),
            filePathsArray = [],
            uploadKeyArray = [],
            addToGridStore = false;

        Ext.each(files, function (file) {

            var fileName = file.name || file.fileName,
                filePath = targetNodePath + '/' + fileName;

            var upload = new Ext.ux.file.Upload({
                fmDirector: treePanel,
                file: file,
                fileSelector: fileSelector,
                id: filePath
            });

            var uploadKey = Tine.Tinebase.uploadManager.queueUpload(upload);

            filePathsArray.push(filePath);
            uploadKeyArray.push(uploadKey);

            addToGridStore = grid.currentFolderNode.id === targetNodeId;

        }, this);

        var params = {
                filenames: filePathsArray,
                type: "file",
                tempFileIds: [],
                forceOverwrite: false
        };
        Tine.Expressodriver.fileRecordBackend.createNodes(params, uploadKeyArray, addToGridStore);
    }
});
/*
 * Tine 2.0
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 */
Ext.ns('Tine.Expressodriver');

/**
 * File grid panel
 *
 * @namespace   Tine.Expressodriver
 * @class       Tine.Expressodriver.NodeGridPanel
 * @extends     Tine.widgets.grid.GridPanel
 *
 * <p>Node Grid Panel</p>
 * <p><pre>
 * </pre></p>
 *
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 *
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressodriver.FileGridPanel
 */
Tine.Expressodriver.NodeGridPanel = Ext.extend(Tine.widgets.grid.GridPanel, {
    /**
     * @cfg filesProperty
     * @type String
     */
    filesProperty: 'files',


    /**
     * config values
     * @private
     */
    header: false,
    border: false,
    deferredRender: false,
    autoExpandColumn: 'name',
    showProgress: true,

    recordClass: Tine.Expressodriver.Model.Node,
    hasDetailsPanel: false,
    evalGrants: true,

    /**
     * grid specific
     * @private
     */
    defaultSortInfo: {field: 'name', direction: 'DESC'},
    gridConfig: {
        autoExpandColumn: 'name',
        enableFileDialog: false,
        enableDragDrop: true,
        ddGroup: 'fileDDGroup'
    },

    ddGroup : 'fileDDGroup',
    currentFolderNode : '/',

    /**
     * inits this cmp
     * @private
     */
    initComponent: function() {
        this.recordProxy = Tine.Expressodriver.fileRecordBackend;

        this.gridConfig.cm = this.getColumnModel();

        this.defaultFilters = [
            {field: 'query', operator: 'contains', value: ''},
            {field: 'path', operator: 'equals', value: this.app.getMainScreen().getWestPanel().getContainerTreePanel().getRootPath()}
        ];

        this.filterToolbar = this.filterToolbar || this.getFilterToolbar();
        this.filterToolbar.getQuickFilterPlugin().criteriaIgnores.push({field: 'path'});

        this.plugins = this.plugins || [];
        this.plugins.push(this.filterToolbar);
        this.plugins.push({
            ptype: 'ux.browseplugin',
            multiple: true,
            scope: this,
            enableFileDialog: false,
            handler: this.onFilesSelect
        });

        Tine.Expressodriver.NodeGridPanel.superclass.initComponent.call(this);
        this.getStore().on('load', this.onLoad);
        Tine.Tinebase.uploadManager.on('update', this.onUpdate);
    },

    initFilterPanel: function() {},

    /**
     * after render handler
     */
    afterRender: function() {
        Tine.Expressodriver.NodeGridPanel.superclass.afterRender.call(this);
        this.action_upload.setDisabled(true);
        this.initDropTarget();
        this.currentFolderNode = this.app.getMainScreen().getWestPanel().getContainerTreePanel().getRootNode();
    },

    /**
     * returns cm
     *
     * @return Ext.grid.ColumnModel
     * @private
     *
     * TODO    add more columns
     */
    getColumnModel: function(){
        var columns = [{
                id: 'tags',
                header: this.app.i18n._('Tags'),
                dataIndex: 'tags',
                width: 50,
                renderer: Tine.Tinebase.common.tagsRenderer,
                sortable: false,
                hidden: true
            }, {
                id: 'name',
                header: this.app.i18n._("Name"),
                width: 70,
                sortable: true,
                dataIndex: 'name',
                renderer: Ext.ux.PercentRendererWithName
            },{
                id: 'size',
                header: this.app.i18n._("Size"),
                width: 40,
                sortable: true,
                dataIndex: 'size',
                renderer: Tine.Tinebase.common.byteRenderer.createDelegate(this, [2, true], 3)
            },{
                id: 'contenttype',
                header: this.app.i18n._("Contenttype"),
                width: 50,
                sortable: true,
                dataIndex: 'contenttype',
                renderer: function(value, metadata, record) {

                    var app = Tine.Tinebase.appMgr.get('Expressodriver');
                    if(record.data.type == 'folder') {
                        return app.i18n._("Folder");
                    }
                    else {
                        return value;
                    }
                }
            },{
                id: 'creation_time',
                header: this.app.i18n._("Creation Time"),
                width: 50,
                sortable: true,
                dataIndex: 'creation_time',
                renderer: Tine.Tinebase.common.dateTimeRenderer,
                hidden: true

            },{
                id: 'created_by',
                header: this.app.i18n._("Created By"),
                width: 50,
                sortable: true,
                dataIndex: 'created_by',
                renderer: Tine.Tinebase.common.usernameRenderer,
                hidden: true

            },{
                id: 'last_modified_time',
                header: this.app.i18n._("Last Modified Time"),
                width: 80,
                sortable: true,
                dataIndex: 'last_modified_time',
                renderer: Tine.Tinebase.common.dateTimeRenderer
            },{
                id: 'last_modified_by',
                header: this.app.i18n._("Last Modified By"),
                width: 50,
                sortable: true,
                dataIndex: 'last_modified_by',
                renderer: Tine.Tinebase.common.usernameRenderer,
                hidden: true
            }
        ];

        return new Ext.grid.ColumnModel({
            defaults: {
                sortable: true,
                resizable: true
            },
            columns: columns
        });
    },

    /**
     * status column renderer
     * @param {string} value
     * @return {string}
     */
    statusRenderer: function(value) {
        return this.app.i18n._hidden(value);
    },

    /**
     * init ext grid panel
     * @private
     */
    initGrid: function() {
        Tine.Expressodriver.NodeGridPanel.superclass.initGrid.call(this);

        if (this.usePagingToolbar) {
           this.initPagingToolbar();
        }
    },

    /**
     * inserts a quota Message when using old Browsers with html4upload
     */
    initPagingToolbar: function() {
        if(!this.pagingToolbar || !this.pagingToolbar.rendered) {
            this.initPagingToolbar.defer(50, this);
            return;
        }
        // old browsers
        if (!((! Ext.isGecko && window.XMLHttpRequest && window.File && window.FileList) || (Ext.isGecko && window.FileReader))) {
            var text = new Ext.Panel({padding: 2, html: String.format(this.app.i18n._('The max. Upload Filesize is {0} MB'), Tine.Tinebase.registry.get('maxFileUploadSize') / 1048576 )});
            this.pagingToolbar.insert(12, new Ext.Toolbar.Separator());
            this.pagingToolbar.insert(12, text);
            this.pagingToolbar.doLayout();
        }
    },

    /**
     * returns filter toolbar -> supress OR filters
     * @private
     */
    getFilterToolbar: function(config) {
        config = config || {};
        var plugins = [];
        if (! Ext.isDefined(this.hasQuickSearchFilterToolbarPlugin) || this.hasQuickSearchFilterToolbarPlugin) {
            this.quickSearchFilterToolbarPlugin = new Tine.widgets.grid.FilterToolbarQuickFilterPlugin();
            plugins.push(this.quickSearchFilterToolbarPlugin);
        }

        return new Tine.widgets.grid.FilterToolbar(Ext.apply(config, {
            app: this.app,
            recordClass: this.recordClass,
            filterModels: this.recordClass.getFilterModel().concat(this.getCustomfieldFilters()),
            defaultFilter: 'query',
            filters: this.defaultFilters || [],
            plugins: plugins
        }));
    },

    /**
     * returns add action / test
     *
     * @return {Object} add action config
     */
    getAddAction: function () {
        return {
            requiredGrant: 'addGrant',
            actionType: 'add',
            text: this.app.i18n._('Upload'),
            handler: this.onFilesSelect,
            disabled: true,
            scope: this,
            plugins: [{
                ptype: 'ux.browseplugin',
                multiple: true,
                enableFileDrop: false,
                disable: true
            }],
            iconCls: this.app.appName + 'IconCls'
        };
    },

    /**
     * init actions with actionToolbar, contextMenu and actionUpdater
     * @private
     */
    initActions: function() {
        this.action_upload = new Ext.Action(this.getAddAction());

        this.action_editFile = new Ext.Action({
            requiredGrant: 'editGrant',
            allowMultiple: false,
            text: this.app.i18n._('Properties'),
            handler: this.onEditFile,
            iconCls: 'action_edit_file',
            disabled: false,
            actionType: 'edit',
            scope: this
        });
        this.action_createFolder = new Ext.Action({
            requiredGrant: 'addGrant',
            actionType: 'reply',
            allowMultiple: true,
            text: this.app.i18n._('Create Folder'),
            handler: this.onCreateFolder,
            iconCls: 'action_create_folder',
            disabled: true,
            scope: this
        });

        this.action_goUpFolder = new Ext.Action({
//            requiredGrant: 'readGrant',
            allowMultiple: true,
            actionType: 'goUpFolder',
            text: this.app.i18n._('Folder Up'),
            handler: this.onLoadParentFolder,
            iconCls: 'action_expressodriver_folder_up',
            disabled: true,
            scope: this
        });

        this.action_download = new Ext.Action({
            requiredGrant: 'readGrant',
            allowMultiple: false,
            actionType: 'download',
            text: this.app.i18n._('Save locally'),
            handler: this.onDownload,
            iconCls: 'action_expressodriver_save_all',
            disabled: true,
            scope: this
        });

        this.action_deleteRecord = new Ext.Action({
            requiredGrant: 'deleteGrant',
            allowMultiple: true,
            singularText: this.app.i18n._('Delete'),
            pluralText: this.app.i18n._('Delete'),
            translationObject: this.i18nDeleteActionText ? this.app.i18n : Tine.Tinebase.translation,
            text: this.app.i18n._('Delete'),
            handler: this.onDeleteRecords,
            disabled: true,
            iconCls: 'action_delete',
            scope: this
        });

        this.contextMenu = Tine.Expressodriver.GridContextMenu.getMenu({
            nodeName: Tine.Expressodriver.Model.Node.getRecordName(),
            actions: ['delete', 'rename', 'download', 'resume', 'pause', 'edit'],
            scope: this,
            backend: 'Expressodriver',
            backendModel: 'Node'
        });

        this.folderContextMenu = Tine.Expressodriver.GridContextMenu.getMenu({
            nodeName: this.app.i18n._(this.app.getMainScreen().getWestPanel().getContainerTreePanel().containerName),
            actions: ['delete', 'rename'],
            scope: this,
            backend: 'Expressodriver',
            backendModel: 'Node'
        });

        this.actionUpdater.addActions(this.contextMenu.items);
        this.actionUpdater.addActions(this.folderContextMenu.items);

        this.actionUpdater.addActions([
           this.action_createFolder,
           this.action_goUpFolder,
           this.action_download,
           this.action_deleteRecord,
           this.action_editFile
       ]);
    },

    /**
     * get the right contextMenu
     */
    getContextMenu: function(grid, row, e) {
        var r = this.store.getAt(row),
            type = r ? r.get('type') : null;

        return type === 'folder' ? this.folderContextMenu : this.contextMenu;
    },

    /**
     * get action toolbar
     *
     * @return {Ext.Toolbar}
     */
    getActionToolbar: function() {
        if (! this.actionToolbar) {
            this.actionToolbar = new Ext.Toolbar({
                defaults: {height: 55},
                items: [{
                    xtype: 'buttongroup',
                    columns: 8,
                    defaults: {minWidth: 60},
                    items: [
                        this.splitAddButton ?
                        Ext.apply(new Ext.SplitButton(this.action_upload), {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top',
                            arrowAlign:'right',
                            menu: new Ext.menu.Menu({
                                items: [],
                                plugins: [{
                                    ptype: 'ux.itemregistry',
                                    key:   'Tine.widgets.grid.GridPanel.addButton'
                                }]
                            })
                        }) :
                        Ext.apply(new Ext.Button(this.action_upload), {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top'
                        }),

                        Ext.apply(new Ext.Button(this.action_editFile), {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top'
                        }),
                        Ext.apply(new Ext.Button(this.action_deleteRecord), {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top'
                        }),
                        Ext.apply(new Ext.Button(this.action_createFolder), {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top'
                        }),
                        Ext.apply(new Ext.Button(this.action_goUpFolder), {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top'
                        }),
                        Ext.apply(new Ext.Button(this.action_download), {
                            scale: 'medium',
                            rowspan: 2,
                            iconAlign: 'top'
                        })
                 ]
                }, this.getActionToolbarItems()]
            });

            if (this.filterToolbar && typeof this.filterToolbar.getQuickFilterField == 'function') {
                this.actionToolbar.add('->', this.filterToolbar.getQuickFilterField());
            }
        }

        return this.actionToolbar;
    },

    /**
     * opens the edit dialog
     */
    onEditFile: function() {
        var sel = this.getGrid().getSelectionModel().getSelections();

        if(sel.length == 1) {
            var record = new Tine.Expressodriver.Model.Node(sel[0].data);
            var window = Tine.Expressodriver.NodeEditDialog.openWindow({record: record});
        }

        window.on('saveAndClose', function() {
            this.getGrid().store.reload();
        }, this);
    },

    /**
     * create folder in current position
     *
     * @param {Ext.Component} button
     * @param {Ext.EventObject} event
     */
    onCreateFolder: function(button, event) {
        var app = this.app,
            nodeName = Tine.Expressodriver.Model.Node.getContainerName();

        Ext.MessageBox.prompt(this.app.i18n._('New Folder'), this.app.i18n._('Please enter the name of the new folder:'), function(_btn, _text) {
            var currentFolderNode = app.getMainScreen().getCenterPanel().currentFolderNode;
            if(currentFolderNode && _btn == 'ok') {
                if (! _text) {
                    Ext.Msg.alert(String.format(this.app.i18n._('No {0} added'), nodeName), String.format(this.app.i18n._('You have to supply a {0} name!'), nodeName));
                    return;
                }
                Ext.MessageBox.wait(_('Please wait'), String.format(_('Creating {0}...' ), nodeName));
                var filename = currentFolderNode.attributes.path + '/' + _text;
                Tine.Expressodriver.fileRecordBackend.createFolder(filename);

            }
        }, this);
    },

    /**
     * delete selected files / folders
     *
     * @param {Ext.Component} button
     * @param {Ext.EventObject} event
     */
    onDeleteRecords: function(button, event) {
        var app = this.app,
            nodeName = '',
            sm = app.getMainScreen().getCenterPanel().selectionModel,
            nodes = sm.getSelections();

        if(nodes && nodes.length) {
            for(var i=0; i<nodes.length; i++) {
                var currNodeData = nodes[i].data;

                if(typeof currNodeData.name == 'object') {
                    nodeName += currNodeData.name.name + '<br />';
                }
                else {
                    nodeName += currNodeData.name + '<br />';
                }
            }
        }

        this.conflictConfirmWin = Tine.widgets.dialog.FileListDialog.openWindow({
            modal: true,
            allowCancel: false,
            height: 180,
            width: 300,
            title: app.i18n._('Do you really want to delete the following files?'),
            text: nodeName,
            scope: this,
            handler: function(button){
                if (nodes && button == 'yes') {
                    this.store.remove(nodes);
                    this.pagingToolbar.refresh.disable();
                    Tine.Expressodriver.fileRecordBackend.deleteItems(nodes);
                }

                for(var i=0; i<nodes.length; i++) {
                    var node = nodes[i];

                    if(node.fileRecord) {
                        var upload = Tine.Tinebase.uploadManager.getUpload(node.fileRecord.get('uploadKey'));
                        upload.setPaused(true);
                        Tine.Tinebase.uploadManager.unregisterUpload(upload.id);
                    }

                }
            }
        }, this);
    },

    /**
     * go up one folder
     *
     * @param {Ext.Component} button
     * @param {Ext.EventObject} event
     */
    onLoadParentFolder: function(button, event) {
        var app = this.app,
            currentFolderNode = app.getMainScreen().getCenterPanel().currentFolderNode;

        if(currentFolderNode && currentFolderNode.parentNode) {
            app.getMainScreen().getCenterPanel().currentFolderNode = currentFolderNode.parentNode;
            currentFolderNode.parentNode.select();
        }
    },

    /**
     * grid row doubleclick handler
     *
     * @param {Tine.Expressodriver.NodeGridPanel} grid
     * @param {} row record
     * @param {Ext.EventObjet} e
     */
    onRowDblClick: function(grid, row, e) {
        //ver
        var app = this.app;
        var rowRecord = grid.getStore().getAt(row);

        if(rowRecord.data.type == 'file') {
            var downloadPath = rowRecord.data.path;
            var downloader = new Ext.ux.file.Download({
                params: {
                    method: 'Expressodriver.downloadFile',
                    requestType: 'HTTP',
                    id: '',
                    path: downloadPath
                }
            }).start();
        }

        else if (rowRecord.data.type == 'folder'){
            var treePanel = app.getMainScreen().getWestPanel().getContainerTreePanel();

            var currentFolderNode = treePanel.getNodeById(rowRecord.id);

            if(currentFolderNode) {
                currentFolderNode.select();
                currentFolderNode.expand();
                app.getMainScreen().getCenterPanel().currentFolderNode = currentFolderNode;
            } else {
                // get ftb path filter
                this.filterToolbar.filterStore.each(function(filter) {
                    var field = filter.get('field');
                    if (field === 'path') {
                        filter.set('value', '');
                        filter.set('value', rowRecord.data);
                        filter.formFields.value.setValue(rowRecord.get('path'));

                        this.filterToolbar.onFiltertrigger();
                        return false;
                    }
                }, this);
            }
        }
    },

    /**
     * on upload failure
     *
     * @private
     */
    onUploadFail: function () {
        Ext.MessageBox.alert(
            _('Upload Failed'),
            _('Could not upload file. Filesize could be too big. Please notify your Administrator. Max upload size:') + ' '
            + Tine.Tinebase.common.byteRenderer(Tine.Tinebase.registry.get('maxFileUploadSize'))
        ).setIcon(Ext.MessageBox.ERROR);

        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            grid = app.getMainScreen().getCenterPanel();
        grid.pagingToolbar.refresh.enable();
    },

    /**
     * on remove handler
     *
     * @param {} button
     * @param {} event
     */
    onRemove: function (button, event) {
        var selectedRows = this.selectionModel.getSelections();
        for (var i = 0; i < selectedRows.length; i += 1) {
            this.store.remove(selectedRows[i]);
            var upload = Tine.Tinebase.uploadManager.getUpload(selectedRows[i].get('uploadKey'));
            upload.setPaused(true);
        }
    },

    /**
     * populate grid store
     *
     * @param {} record
     */
    loadRecord: function (record) {
        if (record && record.get(this.filesProperty)) {
            var files = record.get(this.filesProperty);
            for (var i = 0; i < files.length; i += 1) {
                var file = new Ext.ux.file.Upload.file(files[i]);
                file.set('status', 'complete');
                file.set('nodeRecord', new Tine.Expressodriver.Model.Node(file.data));
                this.store.add(file);
            }
        }
    },

    /**
     * copies uploaded temporary file to target location
     *
     * @param upload  {Ext.ux.file.Upload}
     * @param file  {Ext.ux.file.Upload.file}
     */
    onUploadComplete: function(upload, file) {
        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            grid = app.getMainScreen().getCenterPanel();

        // check if we are responsible for the upload
        if (upload.fmDirector != grid) return;

        // $filename, $type, $tempFileId, $forceOverwrite
        Ext.Ajax.request({
            timeout: 10*60*1000, // Overriding Ajax timeout - important!
            params: {
                method: 'Expressodriver.createNode',
                filename: upload.id,
                type: 'file',
                tempFileId: file.get('id'),
                forceOverwrite: true
            },
            success: grid.onNodeCreated.createDelegate(this, [upload], true),
            failure: grid.onNodeCreated.createDelegate(this, [upload], true)
        });

    },

    /**
     * TODO: move to Upload class or elsewhere??
     * updating fileRecord after creating node
     *
     * @param response
     * @param request
     * @param upload
     */
    onNodeCreated: function(response, request, upload) {
        var record = Ext.util.JSON.decode(response.responseText);

        var fileRecord = upload.fileRecord;
        fileRecord.beginEdit();
        fileRecord.set('contenttype', record.contenttype);
        fileRecord.set('created_by', Tine.Tinebase.registry.get('currentAccount'));
        fileRecord.set('creation_time', record.creation_time);
        fileRecord.set('revision', record.revision);
        fileRecord.set('last_modified_by', record.last_modified_by);
        fileRecord.set('last_modified_time', record.last_modified_time);
        fileRecord.set('name', record.name);
        fileRecord.set('path', record.path);
        fileRecord.set('status', 'complete');
        fileRecord.set('progress', 100);
        fileRecord.commit(false);

        upload.fireEvent('update', 'uploadfinished', upload, fileRecord);

        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            grid = app.getMainScreen().getCenterPanel();

        var allRecordsComplete = true;
        var storeItems = grid.getStore().getRange();
        for(var i=0; i<storeItems.length; i++) {
            if(storeItems[i].get('status') && storeItems[i].get('status') !== 'complete') {
                allRecordsComplete = false;
                break;
            }
        }

        if(allRecordsComplete) {
            grid.pagingToolbar.refresh.enable();
        }
    },

    /**
     * upload new file and add to store
     *
     * @param {ux.BrowsePlugin} fileSelector
     * @param {} e
     */
    onFilesSelect: function (fileSelector, event) {
        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            grid = app.getMainScreen().getCenterPanel(),
            targetNode = grid.currentFolderNode,
            gridStore = grid.store,
            rowIndex = false,
            targetFolderPath = grid.currentFolderNode.attributes.path,
            addToGrid = true,
            dropAllowed = false,
            nodeRecord = null;

        if(event && event.getTarget()) {
            rowIndex = grid.getView().findRowIndex(event.getTarget());
        }


        if(targetNode.attributes) {
            nodeRecord = targetNode.attributes.nodeRecord;
        }

        if(rowIndex !== false && rowIndex > -1) {
            var newTargetNode = gridStore.getAt(rowIndex);
            if(newTargetNode && newTargetNode.data.type == 'folder') {
                targetFolderPath = newTargetNode.data.path;
                addToGrid = false;
                nodeRecord = new Tine.Expressodriver.Model.Node(newTargetNode.data);
            }
        }

        if(!nodeRecord.isDropFilesAllowed()) {
            Ext.MessageBox.alert(
                    _('Upload Failed'),
                    app.i18n._('Putting files in this folder is not allowed!')
            ).setIcon(Ext.MessageBox.ERROR);

            return;
        }

        var files = fileSelector.getFileList();

        if(files.length > 0) {
            grid.pagingToolbar.refresh.disable();
        }

        var filePathsArray = [], uploadKeyArray = [];

        Ext.each(files, function (file) {
            var fileRecord = Tine.Expressodriver.Model.Node.createFromFile(file),
                filePath = targetFolderPath + '/' + fileRecord.get('name');

            fileRecord.set('path', filePath);
            var existingRecordIdx = gridStore.find('name', fileRecord.get('name'));
            if(existingRecordIdx < 0) {
                gridStore.add(fileRecord);
            }

            var upload = new Ext.ux.file.Upload({
                fmDirector: grid,
                file: file,
                fileSelector: fileSelector,
                id: filePath
            });

            var uploadKey = Tine.Tinebase.uploadManager.queueUpload(upload);

            filePathsArray.push(filePath);
            uploadKeyArray.push(uploadKey);

        }, this);

        var params = {
                filenames: filePathsArray,
                type: "file",
                tempFileIds: [],
                forceOverwrite: false
        };
        Tine.Expressodriver.fileRecordBackend.createNodes(params, uploadKeyArray, true);
    },

    /**
     * download file
     *
     * @param {} button
     * @param {} event
     */
    onDownload: function(button, event) {

        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            grid = app.getMainScreen().getCenterPanel(),
            selectedRows = grid.selectionModel.getSelections();

        var fileRow = selectedRows[0];

        var downloadPath = fileRow.data.path;
        var downloader = new Ext.ux.file.Download({
            params: {
                method: 'Expressodriver.downloadFile',
                requestType: 'HTTP',
                id: '',
                path: downloadPath
            }
        }).start();
    },

    /**
     * grid on load handler
     *
     * @param grid
     * @param records
     * @param options
     */
    onLoad: function(store, records, options){
        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            grid = app.getMainScreen().getCenterPanel();

        for(var i=records.length-1; i>=0; i--) {
            var record = records[i];
            if(record.get('type') == 'file' && (!record.get('size') || record.get('size') == 0)) {
                var upload = Tine.Tinebase.uploadManager.getUpload(record.get('path'));

                if(upload) {
                      if(upload.fileRecord && record.get('name') == upload.fileRecord.get('name')) {
                          grid.updateNodeRecord(record, upload.fileRecord);
                          record.afterEdit();
                    }
                }
            }
        }
    },

    /**
     * update grid nodeRecord with fileRecord data
     *
     * @param nodeRecord
     * @param fileRecord
     */
    updateNodeRecord: function(nodeRecord, fileRecord) {
        for(var field in fileRecord.fields) {
            nodeRecord.set(field, fileRecord.get(field));
        }
        nodeRecord.fileRecord = fileRecord;
    },

    /**
     * upload update handler
     *
     * @param change {String} kind of change
     * @param upload {Ext.ux.file.Upload} upload
     * @param fileRecord {file} fileRecord
     *
     */
    onUpdate: function(change, upload, fileRecord) {
        var app = Tine.Tinebase.appMgr.get('Expressodriver'),
            grid = app.getMainScreen().getCenterPanel(),
            rowsToUpdate = grid.getStore().query('name', fileRecord.get('name'));

        if(change == 'uploadstart') {
            Tine.Tinebase.uploadManager.onUploadStart();
        }
        else if(change == 'uploadfailure') {
            grid.onUploadFail();
        }

        if(rowsToUpdate.get(0)) {
            if(change == 'uploadcomplete') {
                grid.onUploadComplete(upload, fileRecord);
            }
            else if(change == 'uploadfinished') {
                rowsToUpdate.get(0).set('size', fileRecord.get('size'));
                rowsToUpdate.get(0).set('contenttype', fileRecord.get('contenttype'));
            }
            rowsToUpdate.get(0).afterEdit();
            rowsToUpdate.get(0).commit(false);
        }
    },

    /**
     * init grid drop target
     *
     * @TODO DRY cleanup
     */
    initDropTarget: function(){
        var ddrow = new Ext.dd.DropTarget(this.getEl(), {
            ddGroup : 'fileDDGroup',

            notifyDrop : function(dragSource, e, data){

                if(data.node && data.node.attributes && !data.node.attributes.nodeRecord.isDragable()) {
                    return false;
                }

                var app = Tine.Tinebase.appMgr.get(Tine.Expressodriver.fileRecordBackend.appName),
                    grid = app.getMainScreen().getCenterPanel(),
                    treePanel = app.getMainScreen().getWestPanel().getContainerTreePanel(),
                    dropIndex = grid.getView().findRowIndex(e.target),
                    target = grid.getStore().getAt(dropIndex),
                    nodes = data.selections ? data.selections : [data.node];

                if(!target || (target.data && target.data.type === 'file')) {
                    return false;
                }

                for(var i=0; i<nodes.length; i++) {
                    if(nodes[i].id == target.id) {
                        return false;
                    }
                }

                var targetNode = treePanel.getNodeById(target.id);
                if(targetNode && targetNode.isAncestor(nodes[0])) {
                    return false;
                }

                Tine.Expressodriver.fileRecordBackend.copyNodes(nodes, target, !e.ctrlKey);
                return true;
            },

            notifyOver : function( dragSource, e, data ) {
                if(data.node && data.node.attributes && !data.node.attributes.nodeRecord.isDragable()) {
                    return false;
                }

                var app = Tine.Tinebase.appMgr.get(Tine.Expressodriver.fileRecordBackend.appName),
                    grid = app.getMainScreen().getCenterPanel(),
                    dropIndex = grid.getView().findRowIndex(e.target),
                    treePanel = app.getMainScreen().getWestPanel().getContainerTreePanel(),
                    target= grid.getStore().getAt(dropIndex),
                    nodes = data.selections ? data.selections : [data.node];

                if(!target || (target.data && target.data.type === 'file')) {
                    return false;
                }

                for(var i=0; i<nodes.length; i++) {
                    if(nodes[i].id == target.id) {
                        return false;
                    }
                }

                var targetNode = treePanel.getNodeById(target.id);
                if(targetNode && targetNode.isAncestor(nodes[0])) {
                    return false;
                }

                return this.dropAllowed;
            }
        });
    },

    /**
     * returns view row class
     */
    getViewRowClass: function(record, index, rowParams, store) {
        return '';
    },
});
/*
 * Tine 2.0
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 */

Ext.ns('Tine.Expressodriver');

/**
 * @namespace Tine.Expressodriver
 * @class Tine.Expressodriver.Application
 * @extends Tine.Tinebase.Application
 */
Tine.Expressodriver.Application = Ext.extend(Tine.Tinebase.Application, {
    /**
     * @return {Boolean}
     */
    init: function () {
        Tine.log.info('Initialize Expressodriver');

        if (! Tine.Tinebase.common.hasRight('run', 'Expressodriver', 'main_screen')) {
            Tine.log.debug('No mainscreen right for Expressodriver');
            this.hasMainScreen = false;
        }
    },

    /**
     * Get translated application title of this application
     *
     * @return {String}
     */
    getTitle : function() {
        return this.i18n.gettext('Expressodriver');
    }
});

/*
 * register additional action for genericpickergridpanel
 */
Tine.widgets.relation.MenuItemManager.register('Expressodriver', 'Node', {
    text: 'Save locally',   // _('Save locally')
    iconCls: 'action_expressodriver_save_all',
    requiredGrant: 'readGrant',
    actionType: 'download',
    allowMultiple: false,
    handler: function(action) {
        var node = action.grid.store.getAt(action.gridIndex).get('related_record');
        var downloadPath = node.path;
        var downloader = new Ext.ux.file.Download({
            params: {
                method: 'Expressodriver.downloadFile',
                requestType: 'HTTP',
                id: '',
                path: downloadPath
            }
        }).start();
    }
});

/**
 * @namespace Tine.Expressodriver
 * @class Tine.Expressodriver.MainScreen
 * @extends Tine.widgets.MainScreen
 */
Tine.Expressodriver.MainScreen = Ext.extend(Tine.widgets.MainScreen, {
    activeContentType: 'Node'
});/*
 * Tine 2.0
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 *
 */

Ext.namespace('Tine.Expressodriver');


/**
 * admin settings panel
 *
 * @namespace   Tine.Expressodriver
 * @class       Tine.Expressodriver.AdminPanel
 * @extends     Tine.widgets.dialog.EditDialog
 *
 * <p>Expressodriver Admin Panel</p>
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>

 *
 * @param       {Object} config
 * @constructor
 * Create a new Tine.Expressodriver.AdminPanel
 */
Tine.Expressodriver.AdminPanel = Ext.extend(Tine.widgets.dialog.EditDialog, {

    appName: 'Expressodriver',
    recordClass: Tine.Expressodriver.Model.Settings,
    recordProxy: Tine.Expressodriver.settingsBackend,
    evalGrants: false,
    storageAdaptersPanel: null,

    updateToolbars: function() {
    },

    getFormItems: function() {
        this.storageAdaptersPanel = new Tine.Expressodriver.ExternalAdapterGridPanel({
            title: this.app.i18n._('External Storage Adapters')
        });

        return {
            xtype: 'tabpanel',
            activeTab: 0,
            border: true,
            items: [{
                title: this.app.i18n._('Default'),
                autoScroll: true,
                border: false,
                frame: true,
                xtype: 'fieldset',
                autoHeight: 'auto',
                items: [
                        {
                            name: 'useCache',
                            fieldLabel: this.app.i18n._('Enable cache for adapters'),
                            xtype: 'checkbox'
                        },
                        {
                            name: 'cacheLifetime',
                            fieldLabel: this.app.i18n._('Cache lifetime'),
                            xtype: 'numberfield',
                            allowDecimals: false,
                            allowNegative: false
                        }
                    ]
                },
                this.storageAdaptersPanel
            ]
        };

    },
     /**
     * executed after record got updated from proxy
     */
    onRecordLoad: function() {
        // interrupt process flow until dialog is rendered
        if (! this.rendered) {
            this.onRecordLoad.defer(250, this);
            return;
        }

        this.window.setTitle(String.format(_('Change settings for application {0}'), this.appName));

        if (this.fireEvent('load', this) !== false) {
            var defaultSettings = this.record.get('default'),
                form = this.getForm();
            form.findField('useCache').setValue(defaultSettings.useCache);
            form.findField('cacheLifetime').setValue(defaultSettings.cacheLifetime);

            this.storageAdaptersPanel.loadAdapters(this.record.get('adapters'))

            form.clearInvalid();
            this.loadMask.hide();
        }
    },

    /**
     * is called from onApplyChanges
     * @param {Boolean} closeWindow
     */
    doApplyChanges: function(closeWindow) {
        // we need to sync record before validating to let (sub) panels have
        // current data of other panels
        this.onRecordUpdate();

        // quit copy mode
        this.copyRecord = false;

        if (this.isValid()) {

            var items = [];
            Ext.each(this.storageAdaptersPanel.getGrid().getStore().data.items, function(item){
                items.push(item.data);
            });
            this.record.set('adapters', items);

            var formData = this.getForm().getFieldValues();
            var defaultData = {
                'useCache': formData.useCache,
                'cacheLifetime': formData.cacheLifetime
            };
            this.record.set('default', defaultData);

            Ext.Ajax.request({
                params: {
                    method: 'Expressodriver.saveSettings',
                    recordData: this.record.data
                },
                scope: this,
                success: function (_result, _request) {
                    //this.record = Ext.util.JSON.decode(_result.responseText);

                    if (!Ext.isFunction(this.window.cascade)) {
                        this.onRecordLoad();
                    }
                    var ticketFn = this.onAfterApplyChanges.deferByTickets(this, [closeWindow]),
                            wrapTicket = ticketFn();

                    this.fireEvent('update', Ext.util.JSON.encode(this.record.data), this.mode, this, ticketFn);
                    wrapTicket();
                },
                failure: this.onRequestFailed,
                timeout: 300000 // 5 minutes

            });


        } else {
            this.saving = false;
            this.loadMask.hide();
            Ext.MessageBox.alert(_('Errors'), this.getValidationErrorMessage());
        }
    },

});

/**
 * Expressodriver admin settings popup
 *
 * @param   {Object} config
 * @return  {Ext.ux.Window}
 */
Tine.Expressodriver.AdminPanel.openWindow = function (config) {
    var window = Tine.WindowFactory.getWindow({
        width: 600,
        height: 400,
        id: 'expressodriver-admin-panel',
        name: 'expressodriver-admin-panel',
        contentPanelConstructor: 'Tine.Expressodriver.AdminPanel',
        contentPanelConstructorConfig: config
    });
    return window;
};/*
 * Tine 2.0
 * external storage adapters grid panel
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 *
 */

Ext.namespace('Tine.Expressodriver', 'Tine.Expressodriver.ExternalAdapter');

/**
 * @namespace Tine.Expressodriver.ExternalAdapter
 * @class Tine.Crm.ExternalAdapter.Model
 * @extends Ext.data.Record
 *
 * external adapters model
 */
Tine.Expressodriver.ExternalAdapter.Model = Tine.Tinebase.data.Record.create([
   {name: 'id', type: 'int'},
   {name: 'name'},
   {name: 'adapter'},
   {name: 'url'},
   {name: 'useEmailAsLoginName', type: 'bool'}
], {
    appName: 'Expressodriver',
    modelName: 'ExternalAdapter',
    idProperty: 'id',
    titleProperty: 'external adapters',
    recordName: 'External Adapter',
    recordsName: 'External Adapters'
});

/**
 * get default data from external adapter
 * @returns {Tine.Expressodriver.ExternalAdapter.Model.getDefaultData.data}
 */
Tine.Expressodriver.ExternalAdapter.Model.getDefaultData = function() {

    var data = {
        id: Tine.Expressodriver.Model.getRandomUnusedId(Ext.StoreMgr.get('ExternalAdapterStore'))
    };

    return data;
};


/**
 * get external adapter store
 *
 * @return {Ext.data.JsonStore}
 */
Tine.Expressodriver.ExternalAdapter.getStore = function() {

    var store = Ext.StoreMgr.get('ExternalAdapterStore');
    if (!store) {

        store =  new Ext.data.JsonStore({
            root: 'results',
            totalProperty: 'totalcount',
            id: 'id',
            fields: Tine.Expressodriver.ExternalAdapter.Model
        });

        Ext.StoreMgr.add('ExternalAdapterStore', store);
    }
    return store;
};

/**
 * set external adapter from backend
 * @type Tine.Tinebase.data.RecordProxy
 */
Tine.Expressodriver.ExternalAdapter.Backend = new Tine.Tinebase.data.RecordProxy({
    appName: 'Expressodriver',
    modelName: 'ExternalAdapter',
    recordClass: Tine.Expressodriver.ExternalAdapter.Model
});


/**
 * @namespace   Tine.Expressodriver.ExternalAdapter
 * @class       Tine.Expressodriver.ExternalAdapter.GridPanel
 * @extends     Tine.
 *
 * external adapters grid panel
 *
 * <p>
 * </p>
 *
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 *
 */
Tine.Expressodriver.ExternalAdapterGridPanel = Ext.extend(Tine.widgets.grid.GridPanel, {

    /**
     * @private
     */
    autoExpandColumn: 'url',

    hasQuickSearchFilterToolbarPlugin: false,

    recordClass: Tine.Expressodriver.ExternalAdapter.Model,

    /**
     * eval grants
     * @cfg {Boolean} evalGrants
     */
    evalGrants: false,

    usePagingToolbar: false,

    disableDeleteActionCheckServiceMap: Ext.emptyFn,

    autoRefreshInterval: 300000,

    stateful: false,


    /**
     * @private
     */
    initComponent: function() {
        this.app = this.app ? this.app : Tine.Tinebase.appMgr.get('Expressodriver');

        this.gridConfig = {
        };

        this.gridConfig.columns = [{
            id: 'name',
            header: this.app.i18n._("Name"),
            width: 150,
            sortable: true,
            dataIndex: 'name'
            }, {
                id: 'adapter',
                header: this.app.i18n._("Adapter"),
                width: 150,
                sortable: true,
                dataIndex: 'adapter',
                renderer: this.adapterRenderer.createDelegate(this)

        },{
                id: 'url',
                header: this.app.i18n._("Url"),
                width: 100,
                sortable: true,
                dataIndex: 'url'
        }, new Ext.ux.grid.CheckColumn({
                header: this.app.i18n._("Use e-mail as login name"),
                width: 55,
                sortable: true,
                dataIndex: 'useEmailAsLoginName'
        })];

        Tine.Expressodriver.ExternalAdapterGridPanel.superclass.initComponent.call(this);

    },
    adapterRenderer: function (adapter) {
        return Tine.Tinebase.widgets.keyfield.Renderer.render('Expressodriver', 'externalDrivers', adapter);
    },

    initStore: function() {

        this.store = Tine.Expressodriver.ExternalAdapter.getStore();

    },

    initFilterPanel: function() {},

    initLayout: function() {
        this.supr().initLayout.call(this);

        this.items.push({
            region : 'north',
            height : 55,
            border : false,
            items  : this.actionToolbar
        });
    },

    loadGridData: function(options) {
        // do nothing here
    },

    loadAdapters: function(items){

        var recordProxy = Tine.Expressodriver.ExternalAdapter.Backend;

        if (Ext.isArray(items)) {
                    Ext.each(items, function(item) {
                        var record = recordProxy.recordReader({responseText: Ext.encode(item)});
                        this.store.addSorted(record);
                    }, this);
            }
    },

    /**
     * on update after edit
     *
     * @param {String|Tine.Tinebase.data.Record} record
     */
    onUpdateRecord: function (record) {
        //Tine.Expressodriver.ExternalAdapterGridPanel.superclass.onUpdateRecord.apply(this, arguments);

        console.log('on update record');
        console.log(record);


        var myRecord = this.store.getById(record.id);

        if (myRecord) {
            // copy values from edited record
            myRecord.beginEdit();
            for (var p in record.data) {
                myRecord.set(p, record.get(p));
            }
            myRecord.endEdit();

        } else {
            this.store.add(record);
        }

    }

});
/*
 * Tine 2.0
 *
 * @package     Expressodriver
 * @license     http://www.gnu.org/licenses/agpl.html AGPL Version 3
 * @copyright   Copyright (c) 2007-2014 Metaways Infosystems GmbH (http://www.metaways.de)
 * @copyright   Copyright (c) 2014 Serpro (http://www.serpro.gov.br)
 * @author      Marcelo Teixeira <marcelo.teixeira@serpro.gov.br>
 * @author      Edgar de Lucca <edgar.lucca@serpro.gov.br>
 */

Ext.ns('Tine.Expressodriver', 'Tine.Expressodriver.ExternalAdapter');

/**
 * @namespace Tine.Expressodriver
 * @class Tine.Expressodriver.ExternalAdapterEditDialog
 * @extends Tine.widgets.dialog.EditDialog
 *
 *
 *
 * @param {Object}
 *            config
 * @constructor Create a new Tine.Expressodriver.ExternalAdapterEditDialog
 */
Tine.Expressodriver.ExternalAdapterEditDialog = Ext.extend(Tine.widgets.dialog.EditDialog, {
    windowNamePrefix: 'ExternalAdapterEditWindow_',
    appName: 'Expressodriver',
    recordClass: Tine.Expressodriver.ExternalAdapter.Model,
    //recordProxy : Tine.Expressodriver.ExternalAdapter.Backend,
    mode: 'local',
    loadRecord: true,
    evalGrants: false,

    /**
     * generic apply changes handler
     */
    onApplyChanges: function() {
        this.onRecordUpdate();

        var form = this.getForm();
        if (form.isValid()) {
            var values = form.getValues();

            this.fireEvent('update', this.record);
            this.window.close();

        } else {
            Ext.MessageBox.alert(_('Errors'), _('Please fix the errors noted.'));
        }
    },
    getFormItems: function() {

        return {
            xtype: 'tabpanel',
            border: false,
            plain: true,
            activeTab: 0,
            border : false,
                    items: [
                        {
                            title: this.app.i18n._('External Adapter'),
                            autoScroll: true,
                            border: false,
                            frame: true,
                            layout: 'border',
                            items: {
                                region: 'center',
                                xtype: 'columnform',
                                labelAlign: 'top',
                                formDefaults: {
                                    xtype: 'textfield',
                                    anchor: '100%',
                                    labelSeparator: '',
                                    columnWidth: .333

                                },
                                items: [
                                    [{
                                            columnWidth: 1,
                                            name: 'name',
                                            fieldLabel: this.app.i18n._('Name'),
                                            allowBlank: false
                                        }], [
                                        new Tine.Tinebase.widgets.keyfield.ComboBox({
                                            app: 'Expressodriver',
                                            keyFieldName: 'externalDrivers',
                                            fieldLabel: this.app.i18n._('Adapter'),
                                            name: 'adapter'
                                        })

                                        ], [{
                                            columnWidth: 1,
                                            name: 'url',
                                            fieldLabel: this.app.i18n._('Url'),
                                            allowBlank: false

                                        }], [{
                                            name: 'useEmailAsLoginName',
                                            fieldLabel: this.app.i18n._('Use e-mail as login name'),
                                            xtype: 'checkbox'
                                        }]
                                ]
                            }

                        }]
        }
    }

});

/**
 * external adapter edit popup
 *
 * @param {Object}
 *            config
 * @return {Ext.ux.Window}
 */
Tine.Expressodriver.ExternalAdapterEditDialog.openWindow = function(config) {
    var id = (config.record && config.record.id) ? config.record.id : 0;
    var window = Tine.WindowFactory.getWindow({
        width: 500,
        height: 400,
        name: Tine.Expressodriver.ExternalAdapterEditDialog.prototype.windowNamePrefix + id,
        contentPanelConstructor: 'Tine.Expressodriver.ExternalAdapterEditDialog',
        contentPanelConstructorConfig: config
    });
    return window;
};
